@@ -1042,7 +1042,6 @@ table_view_contents (GtkWidget *button, BrowserConfig *config)
 	GdaQueryField *field;
 	GObject *sel;
 	GtkWidget *grid, *dlg;
-	gchar *str;
 
 	sel = gnome_db_selector_get_selected_object (GNOME_DB_SELECTOR (config->db_selector));
 	if (!sel || !GDA_IS_DICT_TABLE (sel))
@@ -1053,6 +1052,7 @@ table_view_contents (GtkWidget *button, BrowserConfig *config)
 
 	if (!dlg) {
 		GdaDataModel *query_model;
+		gchar *str;
 
 		/* Query preparation: "SELECT * FROM table" */
 		query = gda_query_new (config->dict);
@@ -1069,279 +1069,46 @@ table_view_contents (GtkWidget *button, BrowserConfig *config)
 		
 		/* Grid */
 		query_model = gda_data_model_query_new (query);
+		gda_data_model_query_compute_modification_queries (GDA_DATA_MODEL_QUERY (query_model),
+								   NULL, 
+								   GDA_DATA_MODEL_QUERY_OPTION_USE_ALL_FIELDS_IF_NO_PK, 
+								   NULL);
+		g_object_unref (query);
+		
+		/* print modification queries */
+		g_object_get (G_OBJECT (query_model), "insert_query", &query, NULL);
+		if (query) {
+			str = gda_renderer_render_as_sql (GDA_RENDERER (query), NULL, 
+							  GDA_RENDERER_EXTRA_VAL_ATTRS, NULL);
+			g_print("===> SQL insert is %s \n", str);
+			g_free (str);
+			g_object_unref (query);
+		}
+		g_object_get (G_OBJECT (query_model), "update_query", &query, NULL);
+		if (query) {
+			str = gda_renderer_render_as_sql (GDA_RENDERER (query), NULL, 
+							  GDA_RENDERER_EXTRA_VAL_ATTRS, NULL);
+			g_print("===> SQL update is %s \n", str);
+			g_free (str);
+			g_object_unref (query);
+		}
+		g_object_get (G_OBJECT (query_model), "delete_query", &query, NULL);
+		if (query) {
+			str = gda_renderer_render_as_sql (GDA_RENDERER (query), NULL, 
+							  GDA_RENDERER_EXTRA_VAL_ATTRS, NULL);
+			g_print("===> SQL delete is %s \n", str);
+			g_free (str);
+			g_object_unref (query);
+		}
+
+		/* grid widget */
+		grid = gnome_db_grid_new (query_model);
+		g_object_unref (query_model);
+
 		if (gda_dict_table_is_view (table)) 
 			str = g_strdup_printf (_("Contents of view '%s'"), gda_object_get_name (GDA_OBJECT (table)));
 		else 
 			str = g_strdup_printf (_("Contents of table '%s'"), gda_object_get_name (GDA_OBJECT (table)));
-
-  		/***************************************************************************************************/		
-		{
-			GdaQuery *query_ins;
-			GdaQueryTarget *target;
-			GSList *table_field_list, *list;
-			gint i = 0;
-			
-			query_ins = gda_query_new (config->dict);
-			gda_query_set_query_type (query_ins, GDA_QUERY_TYPE_INSERT);
-			
-			target = gda_query_target_new (query_ins, gda_object_get_name (GDA_OBJECT (table)));
-			gda_query_add_target (query_ins, target, NULL);
-			
-			table_field_list = gda_entity_get_fields (GDA_ENTITY(table));
-			
-			list = table_field_list;
-			while (list) {
-				GdaDictField *tfield;
-				GdaQueryFieldField *qfieldfield;
-				GdaQueryFieldValue *qfieldvalue;
-				gchar *str;
-
-				tfield = GDA_DICT_FIELD (list->data);
-
-				/* field query field */
-				qfieldfield = g_object_new (GDA_TYPE_QUERY_FIELD_FIELD,
-							    "dict", config->dict,
-							    "query", query_ins,
-							    "target", target,
-							    "field", tfield,
-							    NULL);
-				gda_object_set_name (GDA_OBJECT (qfieldfield), gda_object_get_name (GDA_OBJECT (tfield)));
-				gda_entity_add_field (GDA_ENTITY (query_ins), GDA_ENTITY_FIELD (qfieldfield));
-
-				/* value query field */
-				qfieldvalue = (GdaQueryFieldValue *) gda_query_field_value_new (query_ins, 
-								     gda_entity_field_get_g_type (GDA_ENTITY_FIELD (tfield)));
-				gda_query_field_set_visible (GDA_QUERY_FIELD (qfieldvalue), FALSE);
-				str = g_strdup_printf ("+%d", i++);
-				gda_object_set_name (GDA_OBJECT (qfieldvalue), str);
-				g_free (str);
-				gda_query_field_value_set_is_parameter (GDA_QUERY_FIELD_VALUE (qfieldvalue), TRUE);
-				gda_query_field_value_set_not_null (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-								    !gda_dict_field_is_null_allowed (tfield));
-				gda_query_field_value_set_default_value (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-									 gda_dict_field_get_default_value (tfield));
-				gda_entity_add_field (GDA_ENTITY (query_ins), GDA_ENTITY_FIELD (qfieldvalue));
-				g_object_set (qfieldfield, "value-provider", qfieldvalue, NULL);
-				
-				g_object_unref (G_OBJECT (qfieldvalue));
-				g_object_unref (G_OBJECT (qfieldfield));
-
-				list = g_slist_next (list);
-			}
-
-			g_object_set (query_model, "insert_query", query_ins, NULL);
-			
-			g_object_unref (target);
-			g_print("===> SQL ins is %s \n", gda_renderer_render_as_sql (GDA_RENDERER (query_ins), NULL, 0, NULL) );
-			g_object_unref (query_ins);
-		}
-  		/***************************************************************************************************/		
-		{
-			GdaQuery *query_upd;
-			GdaQueryTarget *target;
-			GSList *table_field_list, *list;
-			gint i = 0;
-			GdaQueryCondition *cond, *newcond;
-			gboolean has_pk;
-			
-			query_upd = gda_query_new (config->dict);
-			gda_query_set_query_type (query_upd, GDA_QUERY_TYPE_UPDATE);
-			
-			target = gda_query_target_new (query_upd, gda_object_get_name (GDA_OBJECT (table)));
-			gda_query_add_target (query_upd, target, NULL);
-
-			cond = gda_query_condition_new(query_upd, GDA_QUERY_CONDITION_NODE_AND);
-
-			has_pk = gda_dict_table_get_pk_constraint (table) ? TRUE : FALSE;
-
-			table_field_list = gda_entity_get_fields (GDA_ENTITY(table));
-			
-			list = table_field_list;
-			while (list) {
-				GdaDictField *tfield;
-				GdaQueryFieldField *qfieldfield;
-				GdaQueryFieldValue *qfieldvalue;
-				gchar *str;
-
-				tfield = GDA_DICT_FIELD (list->data);
-
-				/* field query field */
-				qfieldfield = g_object_new (GDA_TYPE_QUERY_FIELD_FIELD,
-							    "dict", config->dict,
-							    "query", query_upd,
-							    "target", target,
-							    "field", tfield,
-							    NULL);
-				gda_object_set_name (GDA_OBJECT (qfieldfield), gda_object_get_name (GDA_OBJECT (tfield)));
-				gda_entity_add_field (GDA_ENTITY (query_upd), GDA_ENTITY_FIELD (qfieldfield));
-					
-				/* value query field */
-				qfieldvalue = GDA_QUERY_FIELD_VALUE (gda_query_field_value_new (query_upd, 
-												gda_entity_field_get_g_type (GDA_ENTITY_FIELD (tfield))));
-				gda_query_field_set_visible (GDA_QUERY_FIELD (qfieldvalue), FALSE);
-				str = g_strdup_printf ("+%d", i);
-				gda_object_set_name (GDA_OBJECT (qfieldvalue), str);
-				g_free (str);
-				gda_query_field_value_set_is_parameter (GDA_QUERY_FIELD_VALUE (qfieldvalue), TRUE);
-				gda_query_field_value_set_not_null (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-								    !gda_dict_field_is_null_allowed (tfield));
-				gda_query_field_value_set_default_value (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-									 gda_dict_field_get_default_value (tfield));
-				gda_entity_add_field (GDA_ENTITY (query_upd), GDA_ENTITY_FIELD (qfieldvalue));
-				g_object_set (qfieldfield, "value-provider", qfieldvalue, NULL);
-				g_object_unref (G_OBJECT (qfieldvalue));
-
- 				if (!has_pk ||
-				    (has_pk && gda_dict_field_is_pkey_part (tfield))) {
-
-					/* condition value query field */
-					qfieldvalue = GDA_QUERY_FIELD_VALUE (gda_query_field_value_new (query_upd, 
-													gda_entity_field_get_g_type (GDA_ENTITY_FIELD (tfield))));
-					gda_query_field_set_visible (GDA_QUERY_FIELD (qfieldvalue), FALSE);
-					str = g_strdup_printf ("-%d", i);
-					gda_object_set_name (GDA_OBJECT (qfieldvalue), str);
-					g_free (str);
-					gda_entity_add_field (GDA_ENTITY (query_upd), GDA_ENTITY_FIELD (qfieldvalue));
-					gda_query_field_value_set_is_parameter (GDA_QUERY_FIELD_VALUE (qfieldvalue), TRUE);
-					gda_query_field_value_set_not_null (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-									    !gda_dict_field_is_null_allowed (tfield));
-					gda_query_field_value_set_default_value (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-										 gda_dict_field_get_default_value (tfield));
-
-					newcond = gda_query_condition_new(query_upd, GDA_QUERY_CONDITION_LEAF_EQUAL);
-
-					gda_query_condition_leaf_set_operator (newcond, GDA_QUERY_CONDITION_OP_LEFT, GDA_QUERY_FIELD (qfieldfield));
-					gda_query_condition_leaf_set_operator (newcond, GDA_QUERY_CONDITION_OP_RIGHT, GDA_QUERY_FIELD (qfieldvalue));
-
-					gda_query_condition_node_add_child (cond, newcond, NULL);
-
-					g_object_unref (newcond);
- 				
-					g_object_unref (G_OBJECT (qfieldvalue));
-				}
-
-				g_object_unref (G_OBJECT (qfieldfield));
-
-				list = g_slist_next (list);
-				i++;
-			}
-
-			gda_query_set_condition (query_upd, cond);
-			g_object_unref (cond);
-
-			g_object_set (query_model, "update_query", query_upd, NULL);
-
-			g_object_unref (target);
-			g_print("===> SQL upd is %s \n", gda_renderer_render_as_sql (GDA_RENDERER (query_upd), NULL, 0, NULL) );
-
-			{
-				GdaGraphviz *gv;
-				gv = gda_graphviz_new (config->dict);
-				gda_graphviz_add_to_graph  (gv, query_upd);
-				gda_graphviz_save_file (gv, "/tmp/query_upd.dot", NULL);
-				g_object_unref (gv);
-			}
-
-			g_object_unref (query_upd);
-		}
-  		/***************************************************************************************************/		
-		{
-			GdaQuery *query_del;
-			GdaQueryTarget *target;
-			GSList *table_field_list, *list;
-			gint i = 0;
-			GdaQueryCondition *cond, *newcond;
-			gboolean has_pk;
-			
-			query_del = gda_query_new (config->dict);
-			gda_query_set_query_type (query_del, GDA_QUERY_TYPE_DELETE);
-			
-			target = gda_query_target_new (query_del, gda_object_get_name (GDA_OBJECT (table)));
-			gda_query_add_target (query_del, target, NULL);
-
-			cond = gda_query_condition_new(query_del, GDA_QUERY_CONDITION_NODE_AND);
-
-			has_pk = gda_dict_table_get_pk_constraint (table) ? TRUE : FALSE;
-
-			table_field_list = gda_entity_get_fields (GDA_ENTITY(table));
-			
-			list = table_field_list;
-			while (list) {
-				GdaDictField *tfield;
-				GdaQueryFieldField *qfieldfield;
-				GdaQueryFieldValue *qfieldvalue;
-				gchar *str;
-
-				tfield = GDA_DICT_FIELD (list->data);
-
- 				if (!has_pk ||
-				    (has_pk && gda_dict_field_is_pkey_part (tfield))) {
-
-                                        /* field query field */
-                                        qfieldfield = g_object_new (GDA_TYPE_QUERY_FIELD_FIELD,
-								    "dict", config->dict,
-								    "query", query_del,
-								    "target", target,
-								    "field", tfield,
-								    NULL);
-				        gda_object_set_name (GDA_OBJECT (qfieldfield), gda_object_get_name (GDA_OBJECT (tfield)));
-
-					/* condition value query field */
-					qfieldvalue = GDA_QUERY_FIELD_VALUE (gda_query_field_value_new (query_del, 
-													gda_entity_field_get_g_type (GDA_ENTITY_FIELD (tfield))));
-					gda_query_field_set_visible (GDA_QUERY_FIELD (qfieldvalue), FALSE);
-					str = g_strdup_printf ("-%d", i);
-					gda_object_set_name (GDA_OBJECT (qfieldvalue), str);
-					g_free (str);
-					gda_entity_add_field (GDA_ENTITY (query_del), GDA_ENTITY_FIELD (qfieldvalue));
-					gda_query_field_value_set_is_parameter (GDA_QUERY_FIELD_VALUE (qfieldvalue), TRUE);
-					gda_query_field_value_set_not_null (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-									    !gda_dict_field_is_null_allowed (tfield));
-					gda_query_field_value_set_default_value (GDA_QUERY_FIELD_VALUE (qfieldvalue), 
-										 gda_dict_field_get_default_value (tfield));
-				        g_object_set (qfieldfield, "value-provider", qfieldvalue, NULL);
-
-					newcond = gda_query_condition_new(query_del, GDA_QUERY_CONDITION_LEAF_EQUAL);
-
-					gda_query_condition_leaf_set_operator (newcond, GDA_QUERY_CONDITION_OP_LEFT, GDA_QUERY_FIELD (qfieldfield));
-					gda_query_condition_leaf_set_operator (newcond, GDA_QUERY_CONDITION_OP_RIGHT, GDA_QUERY_FIELD (qfieldvalue));
-
-					gda_query_condition_node_add_child (cond, newcond, NULL);
-
-					g_object_unref (newcond);
- 				
-					g_object_unref (G_OBJECT (qfieldvalue));
-
-				        g_object_unref (G_OBJECT (qfieldfield));
-				}
-
-				list = g_slist_next (list);
-				i++;
-			}
-
-			gda_query_set_condition (query_del, cond);
-			g_object_unref (cond);
-
-			g_object_set (query_model, "delete_query", query_del, NULL);
-
-			g_object_unref (target);
-			g_print("===> SQL del is %s \n", gda_renderer_render_as_sql (GDA_RENDERER (query_del), NULL, 0, NULL) );
-
-			{
-				GdaGraphviz *gv;
-				gv = gda_graphviz_new (config->dict);
-				gda_graphviz_add_to_graph  (gv, query_del);
-				gda_graphviz_save_file (gv, "/tmp/query_del.dot", NULL);
-				g_object_unref (gv);
-			}
-
-			g_object_unref (query_del);
-		}
-  		/***************************************************************************************************/		
-
-		grid = gnome_db_grid_new (query_model);
-		g_object_unref (query_model);
-
 		/*g_object_set (G_OBJECT (grid), "title_string", str, NULL);*/
 		dlg = gtk_dialog_new_with_buttons (str, GTK_WINDOW (config->mainwin), 0,
 						   GTK_STOCK_CLOSE,
@@ -1353,7 +1120,6 @@ table_view_contents (GtkWidget *button, BrowserConfig *config)
 		gtk_widget_show_all (dlg);
 		gnome_db_grid_set_sample_size (GNOME_DB_GRID (grid), 300);
 
-		g_object_unref (G_OBJECT (query));
 		g_object_set_data (G_OBJECT (table), "contents", dlg);
 		g_signal_connect (G_OBJECT (dlg), "response",
 				  G_CALLBACK (dialog_exec_response_cb), table);
