@@ -22,6 +22,7 @@
 #define OPTIMIZE 0
 
 #include <glib/gi18n-lib.h>
+#include <string.h>
 #include <stdlib.h>
 #include <libgda/libgda.h>
 #include "gnome-db-data-cell-renderer-textual.h"
@@ -29,6 +30,8 @@
 #include "gnome-db-data-entry.h"
 #include "gnome-db-entry-string.h"
 
+#define MAX_ACCEPTED_STRING_LENGTH 500
+
 static void gnome_db_data_cell_renderer_textual_init       (GnomeDbDataCellRendererTextual      *celltext);
 static void gnome_db_data_cell_renderer_textual_class_init (GnomeDbDataCellRendererTextualClass *class);
 static void gnome_db_data_cell_renderer_textual_dispose    (GObject *object);
@@ -79,10 +82,11 @@ enum {
 
 struct _GnomeDbDataCellRendererTextualPrivate
 {
-	GdaDataHandler   *dh;
-	GType      type;
+	GdaDataHandler *dh;
+	GType           type;
+	gboolean        type_forced; /* TRUE if ->type has been forced by a value and changed from what was specified */
 	GValue         *value;
-	gboolean          to_be_deleted;
+	gboolean        to_be_deleted;
 };
 
 typedef struct 
@@ -131,6 +135,7 @@ gnome_db_data_cell_renderer_textual_init (GnomeDbDataCellRendererTextual *datace
 	datacell->priv = g_new0 (GnomeDbDataCellRendererTextualPrivate, 1);
 	datacell->priv->dh = NULL;
 	datacell->priv->type = G_TYPE_INVALID;
+	datacell->priv->type_forced = FALSE;
 	datacell->priv->value = NULL;
 
 	GTK_CELL_RENDERER (datacell)->xalign = 0.0;
@@ -258,13 +263,37 @@ gnome_db_data_cell_renderer_textual_set_property (GObject *object,
 			if (gval && !gda_value_is_null (gval)) {
 				gchar *str;
 				
-				g_return_if_fail (G_VALUE_TYPE (gval) == datacell->priv->type);
+				if (G_VALUE_TYPE (gval) != datacell->priv->type) {
+					if (!datacell->priv->type_forced) {
+						datacell->priv->type_forced = TRUE;
+						g_warning (_("Data cell renderer's specified type (%s) differs from actual "
+							     "value to display type (%s)"), 
+							   g_type_name (datacell->priv->type),
+							   g_type_name (G_VALUE_TYPE (gval)));
+					}
+					else
+						g_warning (_("Data cell renderer asked to display values of different "
+							     "data types, at least %s and %s, which means the data model has "
+							     "some incoherencies"),
+							   g_type_name (datacell->priv->type),
+							   g_type_name (G_VALUE_TYPE (gval)));
+					datacell->priv->type = G_VALUE_TYPE (gval);
+				}
 
 				if (! OPTIMIZE)
 					datacell->priv->value = gda_value_copy (gval);
 
 				if (datacell->priv->dh) {
 					str = gda_data_handler_get_str_from_value (datacell->priv->dh, gval);
+					if (strlen (str) > MAX_ACCEPTED_STRING_LENGTH) {
+						gchar *msg;
+						gint len;
+						msg = _("<string cut because too long>");
+						len = strlen (msg);
+						str [MAX_ACCEPTED_STRING_LENGTH] = 0;
+						memmove (str + len, str, MAX_ACCEPTED_STRING_LENGTH - len);
+						memcpy (str, msg, strlen (msg));
+					}
 					g_object_set (G_OBJECT (object), "text", str, NULL);
 					g_free (str);
 				}
@@ -280,7 +309,8 @@ gnome_db_data_cell_renderer_textual_set_property (GObject *object,
 		g_object_notify (object, "value");
 		break;
 	case PROP_VALUE_ATTRIBUTES:
-		/* nothing to do */
+		/*if (g_value_get_uint (value) & GDA_VALUE_ATTR_IS_DEFAULT)
+		  g_object_set (G_OBJECT (object), "text", "", NULL);*/
 		break;
 	case PROP_TO_BE_DELETED:
 		datacell->priv->to_be_deleted = g_value_get_boolean (value);
