@@ -1,6 +1,6 @@
 /* gnome-db-entry-wrapper.c
  *
- * Copyright (C) 2003 - 2005 Vivien Malerba
+ * Copyright (C) 2003 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -36,24 +36,26 @@ static void gnome_db_entry_wrapper_get_property (GObject *object,
 						 GParamSpec *pspec);
 
 static void contents_changed_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap);
+static void contents_activated_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap);
 static void check_correct_init (GnomeDbEntryWrapper *wid);
 static void block_signals (GnomeDbEntryWrapper *wid);
 static void unblock_signals (GnomeDbEntryWrapper *wid);
 
 /* GnomeDbDataEntry interface */
 static void            gnome_db_entry_wrapper_data_entry_init   (GnomeDbDataEntryIface *iface);
-static void            gnome_db_entry_wrapper_set_value_type    (GnomeDbDataEntry *de, GdaValueType type);
-static GdaValueType    gnome_db_entry_wrapper_get_value_type    (GnomeDbDataEntry *de);
-static void            gnome_db_entry_wrapper_set_value         (GnomeDbDataEntry *de, const GdaValue *value);
-static GdaValue       *gnome_db_entry_wrapper_get_value         (GnomeDbDataEntry *de);
-static void            gnome_db_entry_wrapper_set_value_orig    (GnomeDbDataEntry *de, const GdaValue *value);
-static const GdaValue *gnome_db_entry_wrapper_get_value_orig    (GnomeDbDataEntry *de);
-static void            gnome_db_entry_wrapper_set_value_default (GnomeDbDataEntry *de, const GdaValue *value);
+static void            gnome_db_entry_wrapper_set_value_type    (GnomeDbDataEntry *de, GType type);
+static GType           gnome_db_entry_wrapper_get_value_type    (GnomeDbDataEntry *de);
+static void            gnome_db_entry_wrapper_set_value         (GnomeDbDataEntry *de, const GValue *value);
+static GValue         *gnome_db_entry_wrapper_get_value         (GnomeDbDataEntry *de);
+static void            gnome_db_entry_wrapper_set_value_orig    (GnomeDbDataEntry *de, const GValue *value);
+static const GValue   *gnome_db_entry_wrapper_get_value_orig    (GnomeDbDataEntry *de);
+static void            gnome_db_entry_wrapper_set_value_default (GnomeDbDataEntry *de, const GValue *value);
 static void            gnome_db_entry_wrapper_set_attributes    (GnomeDbDataEntry *de, guint attrs, guint mask);
 static guint           gnome_db_entry_wrapper_get_attributes    (GnomeDbDataEntry *de);
 static GdaDataHandler *gnome_db_entry_wrapper_get_handler       (GnomeDbDataEntry *de);
 static gboolean        gnome_db_entry_wrapper_expand_in_layout  (GnomeDbDataEntry *de);
-
+static void            gnome_db_entry_wrapper_set_editable      (GnomeDbDataEntry *de, gboolean editable);
+static void            gnome_db_entry_wrapper_grab_focus        (GnomeDbDataEntry *de);
 
 /* properties */
 enum
@@ -63,24 +65,25 @@ enum
 };
 
 struct  _GnomeDbEntryWrapperPriv {
-	gboolean            impl_is_correct;
-        GtkWidget          *entry;
+	gboolean                  impl_is_correct;
+        GtkWidget                *entry;
 	GnomeDbEntryWrapperClass *real_class;
-	guint               signals_blocked;
+	guint                     signals_blocked;
 
-	GdaValueType        type;
-	GdaValue           *value_orig;
-	GdaValue           *value_default; /* Can be of any type, not just 'type' */
+	GType                     type;
+	GValue                   *value_orig;
+	GValue                   *value_default; /* Can be of any type, not just 'type' */
 
-	gboolean            null_forced;
-	gboolean            default_forced;
+	gboolean                  null_forced;
+	gboolean                  default_forced;
 
-	gboolean            null_possible;
-	gboolean            default_possible;
-	gboolean            show_actions;
+	gboolean                  null_possible;
+	gboolean                  default_possible;
+	gboolean                  show_actions;
+	gboolean                  editable;
 
 	/* property */
-	gboolean            set_default_if_invalid;
+	gboolean                  set_default_if_invalid;
 };
 
 /* get a pointer to the parents to be able to call their destructor */
@@ -130,6 +133,8 @@ gnome_db_entry_wrapper_data_entry_init (GnomeDbDataEntryIface *iface)
 	iface->get_attributes = gnome_db_entry_wrapper_get_attributes;
 	iface->get_handler = gnome_db_entry_wrapper_get_handler;
 	iface->expand_in_layout = gnome_db_entry_wrapper_expand_in_layout;
+	iface->set_editable = gnome_db_entry_wrapper_set_editable;
+	iface->grab_focus = gnome_db_entry_wrapper_grab_focus;
 }
 
 
@@ -199,7 +204,8 @@ check_correct_init (GnomeDbEntryWrapper *mgwrap)
 			gtk_widget_show (entry);
 			mgwrap->priv->entry = entry;
 			
-			(*mgwrap->priv->real_class->connect_signals) (mgwrap, G_CALLBACK (contents_changed_cb));
+			(*mgwrap->priv->real_class->connect_signals) (mgwrap, G_CALLBACK (contents_changed_cb),
+								      G_CALLBACK (contents_activated_cb));
 		}
 		else {
 			/* we need to exit because the program WILL BE unstable and WILL crash */
@@ -231,7 +237,7 @@ gnome_db_entry_wrapper_init (GnomeDbEntryWrapper *mgwrap)
 	mgwrap->priv->real_class = NULL;
 	mgwrap->priv->signals_blocked = 0;
 
-	mgwrap->priv->type = GDA_VALUE_TYPE_UNKNOWN;
+	mgwrap->priv->type = G_TYPE_INVALID;
 	mgwrap->priv->value_orig = NULL;
 	mgwrap->priv->value_default = NULL;
 
@@ -241,6 +247,7 @@ gnome_db_entry_wrapper_init (GnomeDbEntryWrapper *mgwrap)
 	mgwrap->priv->null_possible = TRUE;
 	mgwrap->priv->default_possible = FALSE;
 	mgwrap->priv->show_actions = TRUE;
+	mgwrap->priv->editable = TRUE;
 
 	mgwrap->priv->set_default_if_invalid = FALSE;
 }
@@ -251,7 +258,7 @@ gnome_db_entry_wrapper_dispose (GObject *object)
 	GnomeDbEntryWrapper *mgwrap;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_ENTRY_WRAPPER (object));
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (object));
 
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (object);
 
@@ -279,28 +286,28 @@ gnome_db_entry_wrapper_set_property (GObject *object,
 	GnomeDbEntryWrapper *mgwrap = GNOME_DB_ENTRY_WRAPPER (object);
 	if (mgwrap->priv) {
 		switch (param_id) {
-		case PROP_SET_DEFAULT_IF_INVALID:
-			if (mgwrap->priv->set_default_if_invalid != g_value_get_boolean (value)) {
-				guint attrs;
-
-				mgwrap->priv->set_default_if_invalid = g_value_get_boolean (value);
-				attrs = gnome_db_data_entry_get_attributes (GNOME_DB_DATA_ENTRY (mgwrap));
-
-				if (mgwrap->priv->set_default_if_invalid && (attrs & GDA_VALUE_ATTR_DATA_NON_VALID)) {
-					GdaValue *sane_value;
-					GdaDataHandler *dh;
-					GdaValueType type;
-
-					check_correct_init (mgwrap);
-					dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgwrap));
-					type = gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (mgwrap));
-					sane_value = gda_data_handler_get_sane_init_value (dh, type);
-					(*mgwrap->priv->real_class->real_set_value) (mgwrap, sane_value);
+		case PROP_SET_DEFAULT_IF_INVALID: {
+			guint attrs;
+			
+			mgwrap->priv->set_default_if_invalid = g_value_get_boolean (value);
+			attrs = gnome_db_data_entry_get_attributes (GNOME_DB_DATA_ENTRY (mgwrap));
+			
+			if (mgwrap->priv->set_default_if_invalid && (attrs & GDA_VALUE_ATTR_DATA_NON_VALID)) {
+				GValue *sane_value;
+				GdaDataHandler *dh;
+				GType type;
+				
+				check_correct_init (mgwrap);
+				dh = gnome_db_data_entry_get_handler (GNOME_DB_DATA_ENTRY (mgwrap));
+				type = gnome_db_data_entry_get_value_type (GNOME_DB_DATA_ENTRY (mgwrap));
+				sane_value = gda_data_handler_get_sane_init_value (dh, type);
+				(*mgwrap->priv->real_class->real_set_value) (mgwrap, sane_value);
+				if (sane_value)
 					gda_value_free (sane_value);
-				}
 			}
 			break;
 		}
+		}
 	}
 }
 
@@ -320,11 +327,43 @@ gnome_db_entry_wrapper_get_property (GObject *object,
 	}
 }
 
+/**
+ * gnome_db_entry_wrapper_contents_changed
+ * @mgwrap: a #GnomeDbEntryWrapper widget
+ *
+ * Signals to @gwrap that the entry has changed
+ */
+void
+gnome_db_entry_wrapper_contents_changed (GnomeDbEntryWrapper *mgwrap)
+{
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (mgwrap));
+	g_return_if_fail (((GnomeDbEntryWrapper *) mgwrap)->priv);
+
+	contents_changed_cb (NULL, mgwrap);
+}
+
+/**
+ * gnome_db_entry_wrapper_contents_activated
+ * @mgwrap: a #GnomeDbEntryWrapper widget
+ *
+ * Signals to @gwrap that the entry has been activated (that is the user
+ * pressed ENTER for example to signify he has finished entering data)
+ */
+void
+gnome_db_entry_wrapper_contents_activated (GnomeDbEntryWrapper *mgwrap)
+{
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (mgwrap));
+	g_return_if_fail (((GnomeDbEntryWrapper *) mgwrap)->priv);
+
+	contents_activated_cb (NULL, mgwrap);
+}
+
 
 static void gnome_db_entry_wrapper_emit_signal (GnomeDbEntryWrapper *mgwrap);
 static void
 contents_changed_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap)
 {
+	/* @entry is not used */
 	if (! mgwrap->priv->signals_blocked) {
 		mgwrap->priv->null_forced = FALSE;
 		mgwrap->priv->default_forced = FALSE;
@@ -332,6 +371,23 @@ contents_changed_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap)
 	}
 }
 
+static void
+contents_activated_cb (GtkWidget *entry, GnomeDbEntryWrapper *mgwrap)
+{
+	/* @entry is not used */
+	if (! mgwrap->priv->signals_blocked) {
+		mgwrap->priv->null_forced = FALSE;
+		mgwrap->priv->default_forced = FALSE;
+#ifdef debug_signal
+		g_print (">> 'CONTENTS_ACTIVATED' from %s\n", __FUNCTION__);
+#endif
+		g_signal_emit_by_name (G_OBJECT (mgwrap), "contents_activated");
+#ifdef debug_signal
+		g_print ("<< 'CONTENTS_ACTIVATED' from %s\n", __FUNCTION__);
+#endif
+	}
+}
+
 static void
 gnome_db_entry_wrapper_emit_signal (GnomeDbEntryWrapper *mgwrap)
 {
@@ -350,16 +406,16 @@ gnome_db_entry_wrapper_emit_signal (GnomeDbEntryWrapper *mgwrap)
 
 /* Interface implementation */
 static void
-gnome_db_entry_wrapper_set_value_type (GnomeDbDataEntry *iface, GdaValueType type)
+gnome_db_entry_wrapper_set_value_type (GnomeDbDataEntry *iface, GType type)
 {
 	GnomeDbEntryWrapper *mgwrap;
 
-	g_return_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface));
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_if_fail (mgwrap->priv);
 	
 	if (mgwrap->priv->type != type) {
-		GdaValue *value;
+		GValue *value;
 		GdaDataHandler *dh;
 
 		if (mgwrap->priv->value_orig) {
@@ -382,35 +438,35 @@ gnome_db_entry_wrapper_set_value_type (GnomeDbDataEntry *iface, GdaValueType typ
 	}
 }
 
-static GdaValueType
+static GType
 gnome_db_entry_wrapper_get_value_type (GnomeDbDataEntry *iface)
 {
 	GnomeDbEntryWrapper *mgwrap;
 
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface), GDA_VALUE_TYPE_UNKNOWN);
+	g_return_val_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface), G_TYPE_INVALID);
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
-	g_return_val_if_fail (mgwrap->priv, GDA_VALUE_TYPE_UNKNOWN);
+	g_return_val_if_fail (mgwrap->priv, G_TYPE_INVALID);
 
 	return mgwrap->priv->type;
 }
 
 
 static void
-gnome_db_entry_wrapper_set_value (GnomeDbDataEntry *iface, const GdaValue *value)
+gnome_db_entry_wrapper_set_value (GnomeDbDataEntry *iface, const GValue *value)
 {
 	GnomeDbEntryWrapper *mgwrap;
 
-	g_return_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface));
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_if_fail (mgwrap->priv);
 	check_correct_init (mgwrap);
 	
 	block_signals (mgwrap);
 	if (value) {
-		g_return_if_fail ((gda_value_get_type ((GdaValue *) value) == mgwrap->priv->type) || 
-				  (gda_value_get_type ((GdaValue *) value) == GDA_VALUE_TYPE_NULL));
+		g_return_if_fail ((G_VALUE_TYPE ((GValue *) value) == mgwrap->priv->type) || 
+				  (G_VALUE_TYPE ((GValue *) value) == GDA_TYPE_NULL));
 		(*mgwrap->priv->real_class->real_set_value) (mgwrap, value);
-		if (gda_value_is_null ((GdaValue *) value))
+		if (gda_value_is_null ((GValue *) value))
 			mgwrap->priv->null_forced = TRUE;
 		else 
 			mgwrap->priv->null_forced = FALSE;
@@ -425,14 +481,14 @@ gnome_db_entry_wrapper_set_value (GnomeDbDataEntry *iface, const GdaValue *value
 	gnome_db_entry_wrapper_emit_signal (mgwrap);
 }
 
-static GdaValue *
+static GValue *
 gnome_db_entry_wrapper_get_value (GnomeDbDataEntry *iface)
 {
-	GdaValue *value = NULL;
+	GValue *value = NULL;
 	GnomeDbEntryWrapper *mgwrap;
 	GdaDataHandler *dh;
 
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface), NULL);
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_val_if_fail (mgwrap->priv, NULL);
 
@@ -442,14 +498,14 @@ gnome_db_entry_wrapper_get_value (GnomeDbDataEntry *iface)
 		value = gda_value_new_null ();
 	else {
 		if (mgwrap->priv->default_forced) {
-			if (gda_value_get_type (mgwrap->priv->value_default) == mgwrap->priv->type)
+			if (G_VALUE_TYPE (mgwrap->priv->value_default) == mgwrap->priv->type)
 				value = gda_value_copy (mgwrap->priv->value_default);
 			else
 				value = gda_value_new_null ();
 		}
 		else {
 			check_correct_init (mgwrap);
-			value = (*mgwrap->priv->real_class->real_get_value) (mgwrap);
+			value = (mgwrap->priv->real_class->real_get_value) (mgwrap);
 		}
 	}
 
@@ -457,23 +513,27 @@ gnome_db_entry_wrapper_get_value (GnomeDbDataEntry *iface)
 }
 
 static void
-gnome_db_entry_wrapper_set_value_orig (GnomeDbDataEntry *iface, const GdaValue *value)
+gnome_db_entry_wrapper_set_value_orig (GnomeDbDataEntry *iface, const GValue *value)
 {
 	GnomeDbEntryWrapper *mgwrap;
 	gboolean changed = TRUE;
-	GdaValue *evalue;
+	GValue *evalue;
 	
-	g_return_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface));
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_if_fail (mgwrap->priv);
 	check_correct_init (mgwrap);
 
 	/* compare existing value and the one provided as argument */
-	evalue = gnome_db_entry_wrapper_get_value (iface);
-	if (! gda_value_compare_ext ((GdaValue *) value, evalue))
-		changed = FALSE;
-	if (evalue)
-		gda_value_free (evalue);
+	if (mgwrap->priv->real_class->value_is_equal_to)
+		changed = ! mgwrap->priv->real_class->value_is_equal_to (mgwrap, value);
+	else {
+		evalue = gnome_db_entry_wrapper_get_value (iface);
+		if (! gda_value_compare_ext ((GValue *) value, evalue))
+			changed = FALSE;
+		if (evalue)
+			gda_value_free (evalue);
+	}
 
 	/* get rid on any existing orig value */
 	if (mgwrap->priv->value_orig) { 
@@ -489,9 +549,9 @@ gnome_db_entry_wrapper_set_value_orig (GnomeDbDataEntry *iface, const GdaValue *
 	}
 
 	if (value) {
-		g_return_if_fail ((gda_value_get_type ((GdaValue *) value) == mgwrap->priv->type) || 
-				  (gda_value_get_type ((GdaValue *) value) == GDA_VALUE_TYPE_NULL));
-		mgwrap->priv->value_orig = gda_value_copy ((GdaValue *) value);
+		g_return_if_fail ((G_VALUE_TYPE ((GValue *) value) == mgwrap->priv->type) || 
+				  (G_VALUE_TYPE ((GValue *) value) == GDA_TYPE_NULL));
+		mgwrap->priv->value_orig = gda_value_copy ((GValue *) value);
 	}
 	else
 		mgwrap->priv->value_orig = gda_value_new_null ();
@@ -501,21 +561,21 @@ gnome_db_entry_wrapper_set_value_orig (GnomeDbDataEntry *iface, const GdaValue *
 		gnome_db_entry_wrapper_emit_signal (mgwrap);
 }
 
-static const GdaValue *
+static const GValue *
 gnome_db_entry_wrapper_get_value_orig (GnomeDbDataEntry *iface)
 {
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface), NULL);
 	g_return_val_if_fail (GNOME_DB_ENTRY_WRAPPER (iface)->priv, NULL);
 
 	return GNOME_DB_ENTRY_WRAPPER (iface)->priv->value_orig;
 }
 
 static void
-gnome_db_entry_wrapper_set_value_default (GnomeDbDataEntry *iface, const GdaValue *value)
+gnome_db_entry_wrapper_set_value_default (GnomeDbDataEntry *iface, const GValue *value)
 {
 	GnomeDbEntryWrapper *mgwrap;
 
-	g_return_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface));
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_if_fail (mgwrap->priv);
 
@@ -523,12 +583,12 @@ gnome_db_entry_wrapper_set_value_default (GnomeDbDataEntry *iface, const GdaValu
 		gda_value_free (mgwrap->priv->value_default);
 
 	if (value) 
-		mgwrap->priv->value_default = gda_value_copy ((GdaValue *) value);
+		mgwrap->priv->value_default = gda_value_copy ((GValue *) value);
 	else 
 		mgwrap->priv->value_default = gda_value_new_null ();
 
 	if (mgwrap->priv->default_forced) {
-		if (gda_value_get_type (mgwrap->priv->value_default) == mgwrap->priv->type) {
+		if (G_VALUE_TYPE (mgwrap->priv->value_default) == mgwrap->priv->type) {
 			check_correct_init (mgwrap);
 			block_signals (mgwrap);
 			gnome_db_entry_wrapper_set_value (iface, mgwrap->priv->value_default);
@@ -549,7 +609,7 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 {
 	GnomeDbEntryWrapper *mgwrap;
 
-	g_return_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface));
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_if_fail (mgwrap->priv);
 	check_correct_init (mgwrap);
@@ -567,8 +627,8 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 			
 			/* if default is set, see if we can keep it that way */
 			if (mgwrap->priv->default_forced) {
-				if (gda_value_get_type (mgwrap->priv->value_default) != 
-				    GDA_VALUE_TYPE_NULL)
+				if (G_VALUE_TYPE (mgwrap->priv->value_default) != 
+				    GDA_TYPE_NULL)
 					mgwrap->priv->default_forced = FALSE;
 			}
 
@@ -593,7 +653,7 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 		if (attrs & GDA_VALUE_ATTR_IS_DEFAULT) {
 			block_signals (mgwrap);
 			if (mgwrap->priv->value_default) {
-				if (gda_value_get_type (mgwrap->priv->value_default) == mgwrap->priv->type)
+				if (G_VALUE_TYPE (mgwrap->priv->value_default) == mgwrap->priv->type)
 					gnome_db_entry_wrapper_set_value (iface, mgwrap->priv->value_default);
 				else 
 					(*mgwrap->priv->real_class->real_set_value) (mgwrap, NULL);
@@ -604,14 +664,13 @@ gnome_db_entry_wrapper_set_attributes (GnomeDbDataEntry *iface, guint attrs, gui
 
 			/* if NULL is set, see if we can keep it that way */
 			if (mgwrap->priv->null_forced) {
-				if (gda_value_get_type (mgwrap->priv->value_default) != 
-				    GDA_VALUE_TYPE_NULL)
+				if (G_VALUE_TYPE (mgwrap->priv->value_default) != 
+				    GDA_TYPE_NULL)
 					mgwrap->priv->null_forced = FALSE;
 			}
 
 			mgwrap->priv->default_forced = TRUE;
 			gnome_db_entry_wrapper_emit_signal (mgwrap);
-			return;
 		}
 		else {
 			mgwrap->priv->default_forced = FALSE;
@@ -661,23 +720,39 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 {
 	guint retval = 0;
 	GnomeDbEntryWrapper *mgwrap;
-	GdaValue *value;
+	GValue *value = NULL;
+	gboolean has_current_value;
+	gboolean value_is_null = FALSE;
 
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface), 0);
+	g_return_val_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface), 0);
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_val_if_fail (mgwrap->priv, 0);
 
-	value = gnome_db_entry_wrapper_get_value (iface);
-
-	/* NULL? */
-	if (gda_value_get_type (value) == GDA_VALUE_TYPE_NULL) {
-		if (mgwrap->priv->default_forced) {
-			if (mgwrap->priv->null_forced)
-				retval = retval | GDA_VALUE_ATTR_IS_NULL;
-		}
-		else
-			retval = retval | GDA_VALUE_ATTR_IS_NULL;
+	if (!mgwrap->priv->real_class->value_is_equal_to || 
+	    !mgwrap->priv->real_class->value_is_null) {
+		value = gnome_db_entry_wrapper_get_value (iface);
+		has_current_value = TRUE;
 	}
+	else
+		has_current_value = FALSE;
+	
+	/* NULL? */
+	if (has_current_value) {
+		if (G_VALUE_TYPE (value) == GDA_TYPE_NULL) {
+			if (mgwrap->priv->default_forced) {
+				if (mgwrap->priv->null_forced)
+					value_is_null = TRUE;
+			}
+			else
+				value_is_null = TRUE;
+		}
+	}
+	else {
+		if ((mgwrap->priv->real_class->value_is_null) (mgwrap))
+			value_is_null = TRUE;
+	}
+	if (value_is_null)
+		retval = retval | GDA_VALUE_ATTR_IS_NULL;
 
 	/* can be NULL? */
 	if (mgwrap->priv->null_possible) 
@@ -692,15 +767,21 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 		retval = retval | GDA_VALUE_ATTR_CAN_BE_DEFAULT;
 	
 	/* is unchanged */
-	if (mgwrap->priv->value_orig && 
-	    (gda_value_get_type (value) == gda_value_get_type (mgwrap->priv->value_orig))) {
-		if (gda_value_is_null (value)) 
-			retval = retval | GDA_VALUE_ATTR_IS_UNCHANGED;
-		else {
-			if (! gda_value_compare (value, mgwrap->priv->value_orig))
+	if (has_current_value) {
+		if (mgwrap->priv->value_orig && 
+		    (G_VALUE_TYPE (value) == G_VALUE_TYPE (mgwrap->priv->value_orig))) {
+			if (gda_value_is_null (value)) 
 				retval = retval | GDA_VALUE_ATTR_IS_UNCHANGED;
+			else {
+				if (! gda_value_compare (value, mgwrap->priv->value_orig))
+					retval = retval | GDA_VALUE_ATTR_IS_UNCHANGED;
+			}
 		}
 	}
+	else {
+		if ((mgwrap->priv->real_class->value_is_equal_to) (mgwrap, mgwrap->priv->value_orig))
+			retval = retval | GDA_VALUE_ATTR_IS_UNCHANGED;
+	}
 
 	/* actions shown */
 	if (mgwrap->priv->show_actions)
@@ -708,17 +789,20 @@ gnome_db_entry_wrapper_get_attributes (GnomeDbDataEntry *iface)
 
 	/* data valid? */
 	if (! (mgwrap->priv->default_forced && mgwrap->priv->default_possible)) {
-		if (/*(gda_value_is_null (value) && !mgwrap->priv->null_forced) ||*/
-		    (gda_value_is_null (value) && !mgwrap->priv->null_possible))
+		if (/*(value_is_null && !mgwrap->priv->null_forced) ||*/
+		    (value_is_null && !mgwrap->priv->null_possible))
 			retval = retval | GDA_VALUE_ATTR_DATA_NON_VALID;
 	}
 
 	/* has original value? */
-	if (mgwrap->priv->value_orig) {
+	if (mgwrap->priv->value_orig) 
 		retval = retval | GDA_VALUE_ATTR_HAS_VALUE_ORIG;
-	}
 	
-	gda_value_free (value);
+	if (has_current_value)
+		gda_value_free (value);
+
+	if (!mgwrap->priv->editable)
+		retval = retval | GDA_VALUE_ATTR_NO_MODIF;
 
 	return retval;
 }
@@ -730,7 +814,7 @@ gnome_db_entry_wrapper_get_handler (GnomeDbDataEntry *iface)
 	GValue val = { 0, };
 	GdaDataHandler *dh;
 
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface), NULL);
 	g_return_val_if_fail (GNOME_DB_ENTRY_WRAPPER (iface)->priv, NULL);
 
 	g_value_init (&val, G_TYPE_POINTER);
@@ -746,10 +830,45 @@ gnome_db_entry_wrapper_expand_in_layout (GnomeDbDataEntry *iface)
 {
 	GnomeDbEntryWrapper *mgwrap;
 
-	g_return_val_if_fail (iface && IS_GNOME_DB_ENTRY_WRAPPER (iface), FALSE);
+	g_return_val_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface), FALSE);
 	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
 	g_return_val_if_fail (mgwrap->priv, FALSE);
 	check_correct_init (mgwrap);
 
-	return (*mgwrap->priv->real_class->expand_in_layout) (mgwrap);
+	return (mgwrap->priv->real_class->expand_in_layout) (mgwrap);
+}
+
+static void
+gnome_db_entry_wrapper_set_editable (GnomeDbDataEntry *iface, gboolean editable)
+{
+	GnomeDbEntryWrapper *mgwrap;
+
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
+	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
+	g_return_if_fail (mgwrap->priv);
+	check_correct_init (mgwrap);
+
+	mgwrap->priv->editable = editable;
+	if (mgwrap->priv->real_class->set_editable)
+		(mgwrap->priv->real_class->set_editable) (mgwrap, editable);
+	else
+		gtk_widget_set_sensitive (GTK_WIDGET (iface), editable);
+}
+
+static void
+gnome_db_entry_wrapper_grab_focus (GnomeDbDataEntry *iface)
+{
+	GnomeDbEntryWrapper *mgwrap;
+
+	g_return_if_fail (GNOME_DB_IS_ENTRY_WRAPPER (iface));
+	mgwrap = GNOME_DB_ENTRY_WRAPPER (iface);
+	g_return_if_fail (mgwrap->priv);
+	check_correct_init (mgwrap);
+
+	if (mgwrap->priv->real_class->grab_focus)
+		(mgwrap->priv->real_class->grab_focus) (mgwrap);
+	else {
+		if (mgwrap->priv->entry && GTK_WIDGET_CAN_FOCUS (mgwrap->priv->entry))
+			gtk_widget_grab_focus (mgwrap->priv->entry);
+	}
 }
