@@ -1,6 +1,6 @@
 /* gnome-db-data-widget.c
  *
- * Copyright (C) 2004 - 2005 Vivien Malerba
+ * Copyright (C) 2004 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,14 +19,18 @@
  */
 
 #include <string.h>
-#include "gnome-db-base.h"
-#include "gnome-db-dict.h"
-#include "gnome-db-data-set.h"
-#include "gnome-db-qfield.h"
-#include "gnome-db-entity.h"
-#include "gnome-db-parameter.h"
+#include <libgda/libgda.h>
 #include "gnome-db-data-widget.h"
+#include  <glib/gi18n-lib.h>
 
+/* signals */
+enum
+{
+        PROXY_CHANGED,
+        LAST_SIGNAL
+};
+
+static gint gnome_db_data_widget_signals[LAST_SIGNAL] = { 0 };
 
 static void gnome_db_data_widget_iface_init (gpointer g_class);
 
@@ -61,27 +65,36 @@ gnome_db_data_widget_iface_init (gpointer g_class)
 	static gboolean initialized = FALSE;
 
 	if (! initialized) {
+		gnome_db_data_widget_signals[PROXY_CHANGED] = 
+			g_signal_new ("proxy_changed",
+                                      GNOME_DB_TYPE_DATA_WIDGET,
+                                      G_SIGNAL_RUN_FIRST,
+                                      G_STRUCT_OFFSET (GnomeDbDataWidgetIface, proxy_changed),
+                                      NULL, NULL,
+                                      g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE,
+                                      1, G_TYPE_POINTER);
+
 		initialized = TRUE;
 	}
 }
 
 /**
- * gnome_db_data_widget_set_mode
+ * gnome_db_data_widget_get_proxy
  * @iface: an object which implements the #GnomeDbDataWidget interface
- * @mode: an OR'ed collection of flags from #GnomeDbActionMode
- * 
- * Changes the mode that was set when gnome_db_data_widget_run() was called.
+ *
+ * Get a pointer to the #GdaDataProxy being used by @iface
+ *
+ * Returns: a #GdaDataProxy pointer
  */
-void
-gnome_db_data_widget_set_mode (GnomeDbDataWidget *iface, guint mode)
+GdaDataProxy *
+gnome_db_data_widget_get_proxy (GnomeDbDataWidget *iface)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
-
-	if (mode & GNOME_DB_ACTION_MODIF_COMMIT_IMMEDIATE)
-		mode = mode | GNOME_DB_ACTION_MODIF_AUTO_COMMIT;
-
-	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_mode)
-		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_mode) (iface, mode);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_WIDGET (iface), NULL);
+	
+	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_proxy)
+		return (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_proxy) (iface);
+	else
+		return NULL;
 }
 
 /**
@@ -94,7 +107,7 @@ gnome_db_data_widget_set_mode (GnomeDbDataWidget *iface, guint mode)
 void
 gnome_db_data_widget_column_show (GnomeDbDataWidget *iface, gint column)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
+	g_return_if_fail (GNOME_DB_IS_DATA_WIDGET (iface));
 
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->col_set_show)
 		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->col_set_show) (iface, column, TRUE);
@@ -111,7 +124,7 @@ gnome_db_data_widget_column_show (GnomeDbDataWidget *iface, gint column)
 void
 gnome_db_data_widget_column_hide (GnomeDbDataWidget *iface, gint column)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
+	g_return_if_fail (GNOME_DB_IS_DATA_WIDGET (iface));
 
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->col_set_show)
 		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->col_set_show) (iface, column, FALSE);
@@ -129,7 +142,7 @@ gnome_db_data_widget_column_hide (GnomeDbDataWidget *iface, gint column)
 void 
 gnome_db_data_widget_column_set_editable (GnomeDbDataWidget *iface, gint column, gboolean editable)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
+	g_return_if_fail (GNOME_DB_IS_DATA_WIDGET (iface));
 
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_column_editable)
 		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_column_editable) (iface, column, editable);	
@@ -138,7 +151,7 @@ gnome_db_data_widget_column_set_editable (GnomeDbDataWidget *iface, gint column,
 /**
  * gnome_db_data_widget_column_show_actions
  * @iface: an object which implements the #GnomeDbDataWidget interface
- * @column: column number of the data
+ * @column: column number of the data, or -1 to apply the setting to all the columns
  * @show_actions:
  * 
  * Sets if the data entry in the @iface widget at @column (in the data model @iface operates on) must show its
@@ -147,31 +160,12 @@ gnome_db_data_widget_column_set_editable (GnomeDbDataWidget *iface, gint column,
 void
 gnome_db_data_widget_column_show_actions (GnomeDbDataWidget *iface, gint column, gboolean show_actions)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
+	g_return_if_fail (GNOME_DB_IS_DATA_WIDGET (iface));
 
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->show_column_actions)
 		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->show_column_actions) (iface, column, show_actions);
 }
 
-
-/**
- * gnome_db_data_widget_global_show_actions
- * @iface: an object which implements the #GnomeDbDataWidget interface
- * @show_actions:
- * 
- * Controls if the action buttons provided by the @iface widget (for instance SAVE, RESET, MOVE TO NEXT, etc)
- * are displayed or not.
- */
-void
-gnome_db_data_widget_global_show_actions (GnomeDbDataWidget *iface, gboolean show_actions)
-{
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
-
-	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->show_global_actions)
-		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->show_global_actions) (iface, show_actions);
-}
-
-
 /**
  * gnome_db_data_widget_get_actions_group
  * @iface: an object which implements the #GnomeDbDataWidget interface
@@ -193,7 +187,7 @@ gnome_db_data_widget_global_show_actions (GnomeDbDataWidget *iface, gboolean sho
 GtkActionGroup *
 gnome_db_data_widget_get_actions_group (GnomeDbDataWidget *iface)
 {
-	g_return_val_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_WIDGET (iface), NULL);
 
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_actions_group)
 		return (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_actions_group) (iface);
@@ -217,7 +211,7 @@ gnome_db_data_widget_perform_action (GnomeDbDataWidget *iface, GnomeDbAction act
 	GtkActionGroup *group;
 	GtkAction *gtkaction;
 
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
+	g_return_if_fail (GNOME_DB_IS_DATA_WIDGET (iface));
 	group = gnome_db_data_widget_get_actions_group (iface);
 	if (!group) {
 		g_warning ("Object class %s does not support the gnome_db_data_widget_get_actions_group() method",
@@ -275,39 +269,19 @@ gnome_db_data_widget_perform_action (GnomeDbDataWidget *iface, GnomeDbAction act
 }
 
 /**
- * gnome_db_data_widget_get_params
+ * gnome_db_data_widget_get_current_data
  * @iface: an object which implements the #GnomeDbDataWidget interface
  *
- * Get the #GnomeDbDataSet object which contains all the parameters which control the contents
- * of the data in the @iface widget. Modifying the values of the parameters in the returned data
- * set will make the @iface widget refresh its contents.
- *
- * Returns: the #GnomeDbDataSet object for parameters (not a new object)
- */
-GnomeDbDataSet *
-gnome_db_data_widget_get_params (GnomeDbDataWidget *iface)
-{
-	g_return_val_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface), NULL);
-
-	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_params_set)
-		return (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_params_set) (iface);
-	return NULL;
-}
-
-/**
- * gnome_db_data_widget_get_data
- * @iface: an object which implements the #GnomeDbDataWidget interface
- *
- * Get the #GnomeDbDataSet object which contains all the parameters which in turn contain the actual
+ * Get the #GdaDataModelIter object which contains all the parameters which in turn contain the actual
  * data stored in @iface. When the user changes what's displayed or what's selected (depending on the
  * actual widget) in @iface, then the parameter's values change as well.
  *
- * Returns: the #GnomeDbDataSet object for data (not a new object)
+ * Returns: the #GdaParameterList object for data (not a new object)
  */
-GnomeDbDataSet *
-gnome_db_data_widget_get_data (GnomeDbDataWidget *iface)
+GdaDataModelIter *
+gnome_db_data_widget_get_current_data (GnomeDbDataWidget *iface)
 {
-	g_return_val_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_WIDGET (iface), NULL);
 
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_data_set)
 		return (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_data_set) (iface);
@@ -325,7 +299,7 @@ gnome_db_data_widget_get_data (GnomeDbDataWidget *iface)
 GdaDataModel *
 gnome_db_data_widget_get_gda_model (GnomeDbDataWidget *iface)
 {
-	g_return_val_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface), NULL);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_WIDGET (iface), NULL);
 
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_gda_model)
 		return (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_gda_model) (iface);
@@ -340,59 +314,52 @@ gnome_db_data_widget_get_gda_model (GnomeDbDataWidget *iface)
  * Sets the data model which is used by @iface.
  */
 void
-gnome_db_data_widget_set_gda_model (GnomeDbDataWidget *iface, GnomeDbDict *dict, GdaDataModel *model)
+gnome_db_data_widget_set_gda_model (GnomeDbDataWidget *iface, GdaDataModel *model)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
-	g_return_if_fail (!dict || IS_GNOME_DB_DICT (dict));
+	g_return_if_fail (GNOME_DB_IS_DATA_WIDGET (iface));
 	g_return_if_fail (model && GDA_IS_DATA_MODEL (model));
 	
 	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_gda_model)
-		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_gda_model) (iface, dict, model);
+		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_gda_model) (iface, model);
 }
 
 /**
+ * gnome_db_data_widget_set_write_mode
  * @iface: an object which implements the #GnomeDbDataWidget interface
- * @title: a string
+ * @mode:
  *
- * Sets @iface's title
+ * Specifies the way the modifications stored in the #GdaDataProxy used internally by @iface are written back to
+ * the #GdaDataModel which holds the data displayed in @iface.
+ *
+ * Returns: TRUE if the proposed mode has been taken into account
  */
-void
-gnome_db_data_widget_set_title (GnomeDbDataWidget *iface, const gchar *title)
+gboolean
+gnome_db_data_widget_set_write_mode (GnomeDbDataWidget *iface, GnomeDbDataWidgetWriteMode mode)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
-
-	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_title)
-		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_title) (iface, title);
+	g_return_val_if_fail (GNOME_DB_IS_DATA_WIDGET (iface), FALSE);
+	
+	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_write_mode)
+		return (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_write_mode) (iface, mode);
+	else 
+		return FALSE;
 }
 
 /**
+ * gnome_db_data_widget_get_write_mode
  * @iface: an object which implements the #GnomeDbDataWidget interface
- * @show:
  *
- * Tells if @iface's title must be shown or hidden (hidden by default)
- */
-void            
-gnome_db_data_widget_set_show_title (GnomeDbDataWidget *iface, gboolean show)
-{
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
-
-	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_show_title)
-		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_show_title) (iface, show);	
-}
-
-/**
- * @iface: an object which implements the #GnomeDbDataWidget interface
- * @stock_id: the id of the requested icon (like for example "GTK_STOCK_FIND"), or %NULL
- * @size: the icon's size
+ * Get the way the modifications stored in the #GdaDataProxy used internally by @iface are written back to
+ * the #GdaDataModel which holds the data displayed in @iface.
  *
- * Sets @iface's title icon usign a stock icon. If @stock_id is %NULL, then the title won't have an
- * icon anymore
+ * Returns: the write mode used by @iface
  */
-void
-gnome_db_data_widget_set_title_icon_from_stock (GnomeDbDataWidget *iface, const gchar *stock_id, GtkIconSize size)
+GnomeDbDataWidgetWriteMode
+gnome_db_data_widget_get_write_mode (GnomeDbDataWidget *iface)
 {
-	g_return_if_fail (iface && IS_GNOME_DB_DATA_WIDGET (iface));
-
-	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_t_i_from_stock)
-		(GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->set_t_i_from_stock) (iface, stock_id, size);	
+	g_return_val_if_fail (GNOME_DB_IS_DATA_WIDGET (iface), GNOME_DB_DATA_WIDGET_WRITE_ON_DEMAND);
+	
+	if (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_write_mode)
+		return (GNOME_DB_DATA_WIDGET_GET_IFACE (iface)->get_write_mode) (iface);
+	else 
+		return GNOME_DB_DATA_WIDGET_WRITE_ON_DEMAND;
 }
