@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2005 The GNOME Foundation
+ * Copyright (C) 1999 - 2006 The GNOME Foundation
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -119,7 +119,7 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 	table = gnome_db_new_table_widget (7, 3, FALSE);
 	gtk_box_pack_start (GTK_BOX (config), table, TRUE, TRUE, 0);
 
-	str = g_strdup_printf ("<b>%s</b>", _("Data source _name:"));
+	str = g_strdup_printf ("%s <span foreground='red' weight='bold'>*</span>", _("Data source _name:"));
 	label = gtk_label_new ("");
 	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str);
 	g_free (str);
@@ -146,7 +146,7 @@ gnome_db_dsn_config_init (GnomeDbDsnConfig *config, GnomeDbDsnConfigClass *klass
 	gtk_table_attach (GTK_TABLE (table), config->priv->is_global, 1, 2, 1, 2,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
-	str = g_strdup_printf ("<b>%s</b>", _("_Provider:"));
+	str = g_strdup_printf ("%s <span foreground='red' weight='bold'>*</span>", _("_Provider:"));
 	label = gtk_label_new ("");
 	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str);
 	g_free (str);
@@ -281,11 +281,6 @@ gnome_db_dsn_config_get_info (GnomeDbDsnConfig *config)
 
 	dsn_info = g_new0 (GdaDataSourceInfo, 1);
 	dsn_info->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wname)));
-	for (i = 0; i < strlen (dsn_info->name); i++) {
-		if (!g_ascii_isalnum (dsn_info->name[i]))
-			dsn_info->name[i] = '_';
-	}
-
 	dsn_info->provider = g_strdup (gnome_db_provider_selector_get_selected_provider 
 				       (GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
 	dsn_info->cnc_string = gnome_db_dsn_spec_get_specs (GNOME_DB_DSN_SPEC (config->priv->dsn_spec));
