@@ -350,7 +350,7 @@ params_to_string (GnomeDbDsnSpec *spec)
 	GSList *list;
 
 	g_assert (spec->priv->form);
-	if (! IS_GNOME_DB_BASIC_FORM (spec->priv->form))
+	if (! GNOME_DB_IS_BASIC_FORM (spec->priv->form))
 		return NULL;
 
 	dset = gnome_db_basic_form_get_data_set (GNOME_DB_BASIC_FORM (spec->priv->form));
