@@ -29,6 +29,8 @@
 
 #define PARENT_TYPE GTK_TYPE_BIN
 
+
+
 struct _GnomeDbGrayBarPrivate {
 	GtkWidget *hbox;
 	GtkWidget *icon;
@@ -60,6 +62,7 @@ static void gnome_db_gray_bar_get_property (GObject *object,
 					    GParamSpec *pspec);
 static void gnome_db_gray_bar_finalize     (GObject *object);
 
+/* Properties */
 enum {
 	PROP_0,
 	PROP_TEXT,
@@ -231,12 +234,13 @@ gnome_db_gray_bar_class_init (GnomeDbGrayBarClass *klass)
 	/* add class properties */
 	g_object_class_install_property (
 		object_class, PROP_TEXT,
-		g_param_spec_string ("text", NULL, NULL, NULL,
+		g_param_spec_string ("text", NULL, "Text showed inside the widget.", NULL,
 				     (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (
 		object_class, PROP_SHOW_ICON,
-		g_param_spec_string ("show_icon", NULL, NULL, NULL,
+		g_param_spec_boolean ("show_icon", NULL, NULL, FALSE,
 				     (G_PARAM_READABLE | G_PARAM_WRITABLE)));
+	
 }
 
 static void
@@ -283,6 +287,9 @@ gnome_db_gray_bar_set_property (GObject *object,
 	case PROP_TEXT :
 		gnome_db_gray_bar_set_text (bar, g_value_get_string (value));
 		break;
+	case PROP_SHOW_ICON:
+		gnome_db_gray_bar_set_show_icon (bar, g_value_get_boolean(value));
+		break;
 	default :
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -303,6 +310,9 @@ gnome_db_gray_bar_get_property (GObject *object,
 	case PROP_TEXT :
 		g_value_set_string (value, gnome_db_gray_bar_get_text (bar));
 		break;
+	case PROP_SHOW_ICON:
+		g_value_set_boolean(value, gnome_db_gray_bar_get_show_icon (bar));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
