@@ -205,7 +205,7 @@ gnome_db_grid_get_property (GObject *object,
 GList *
 gnome_db_grid_get_selection (GnomeDbGrid *grid)
 {
-	g_return_val_if_fail (grid && IS_GNOME_DB_GRID (grid), NULL);
+	g_return_val_if_fail (grid && GNOME_DB_IS_GRID (grid), NULL);
 	g_return_val_if_fail (grid->priv, NULL);
 
 	return gnome_db_raw_grid_get_selection (GNOME_DB_RAW_GRID (grid->priv->raw_grid));
@@ -221,7 +221,7 @@ gnome_db_grid_get_selection (GnomeDbGrid *grid)
 void
 gnome_db_grid_set_sample_size (GnomeDbGrid *grid, gint sample_size)
 {
-	g_return_if_fail (grid && IS_GNOME_DB_GRID (grid));
+	g_return_if_fail (grid && GNOME_DB_IS_GRID (grid));
 	g_return_if_fail (grid->priv);
 
 	gnome_db_raw_grid_set_sample_size (GNOME_DB_RAW_GRID (grid->priv->raw_grid), sample_size);
