@@ -40,10 +40,15 @@ static void gnome_db_raw_form_get_property (GObject *object,
 
 static void gnome_db_raw_form_initialize (GnomeDbRawForm *form, GtkWidget *layout, GHashTable *box_widgets);
 
+static gboolean iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form);
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form);
 static void proxy_changed_cb (GdaDataProxy *proxy, GnomeDbRawForm *form);
 static void proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GnomeDbRawForm *form);
 
+/* errors reporting */
+static GtkWidget *create_data_error_dialog (GnomeDbRawForm *form, gboolean with_question, GError *filled_error);
+static gboolean display_error_with_keep_or_discard_choice (GnomeDbRawForm *form, GError *filled_error);
+
 /* GnomeDbDataWidget interface */
 static void            gnome_db_raw_form_widget_init         (GnomeDbDataWidgetIface *iface);
 static GdaDataProxy   *gnome_db_raw_form_get_proxy           (GnomeDbDataWidget *iface);
@@ -54,15 +59,18 @@ static GtkActionGroup *gnome_db_raw_form_get_actions_group   (GnomeDbDataWidget
 static GdaDataModelIter *gnome_db_raw_form_widget_get_data_set   (GnomeDbDataWidget *iface);
 static GdaDataModel   *gnome_db_raw_form_widget_get_gda_model (GnomeDbDataWidget *iface);
 static void            gnome_db_raw_form_widget_set_gda_model (GnomeDbDataWidget *iface, GdaDataModel *model);
-
+static gboolean        gnome_db_raw_form_widget_set_write_mode (GnomeDbDataWidget *iface, GnomeDbDataWidgetWriteMode mode);
+static GnomeDbDataWidgetWriteMode gnome_db_raw_form_widget_get_write_mode (GnomeDbDataWidget *iface);
 
 struct _GnomeDbRawFormPriv
 {
-	GdaDataModel      *model;
-	GdaDataProxy      *proxy; /* proxy for @model */
-	GdaDataModelIter  *iter;  /* proxy's iter */
+	GdaDataModel               *model;
+	GdaDataProxy               *proxy; /* proxy for @model */
+	GdaDataModelIter           *iter;  /* proxy's iter */
 
-	GtkActionGroup    *actions_group;
+	GnomeDbDataWidgetWriteMode  write_mode;
+
+	GtkActionGroup             *actions_group;
 };
 
 #define PAGE_NO_DATA 0
@@ -120,9 +128,10 @@ gnome_db_raw_form_widget_init (GnomeDbDataWidgetIface *iface)
 	iface->get_data_set = gnome_db_raw_form_widget_get_data_set;
 	iface->get_gda_model = gnome_db_raw_form_widget_get_gda_model;
 	iface->set_gda_model = gnome_db_raw_form_widget_set_gda_model;
+	iface->set_write_mode = gnome_db_raw_form_widget_set_write_mode;
+	iface->get_write_mode = gnome_db_raw_form_widget_get_write_mode;
 }
 
-
 static void
 gnome_db_raw_form_class_init (GnomeDbRawFormClass * class)
 {
@@ -135,7 +144,7 @@ gnome_db_raw_form_class_init (GnomeDbRawFormClass * class)
         object_class->set_property = gnome_db_raw_form_set_property;
         object_class->get_property = gnome_db_raw_form_get_property;
 	g_object_class_install_property (object_class, PROP_MODEL,
-					 g_param_spec_pointer ("model", _("Data to display"), NULL,
+					 g_param_spec_object ("model", _("Data to display"), NULL, GDA_TYPE_DATA_MODEL,
                                                                G_PARAM_READABLE | G_PARAM_WRITABLE));
 }
 
@@ -150,8 +159,8 @@ static void action_next_record_cb (GtkAction *action, GnomeDbRawForm *form);
 static void action_last_record_cb (GtkAction *action, GnomeDbRawForm *form);
 
 static GtkActionEntry ui_actions[] = {
-	{ "ActionNew", GTK_STOCK_NEW, "_New", NULL, "Create a new data entry", G_CALLBACK (action_new_cb)},
-	{ "ActionDelete", GTK_STOCK_DELETE, "_Delete", NULL, "Delete the selected entry", G_CALLBACK (action_delete_cb)},
+	{ "ActionNew", GTK_STOCK_ADD, "_New", NULL, "Create a new data entry", G_CALLBACK (action_new_cb)},
+	{ "ActionDelete", GTK_STOCK_REMOVE, "_Delete", NULL, "Delete the selected entry", G_CALLBACK (action_delete_cb)},
 	{ "ActionUndelete", GTK_STOCK_UNDELETE, "_Undelete", NULL, "Cancels the deletion of the current entry", 
 	  G_CALLBACK (action_undelete_cb)},	
 	{ "ActionCommit", GTK_STOCK_SAVE, "_Commit", NULL, "Commit the latest changes", G_CALLBACK (action_commit_cb)},
@@ -167,17 +176,75 @@ static GtkActionEntry ui_actions[] = {
 };
 
 static void
-gnome_db_raw_form_init (GnomeDbRawForm * wid)
+action_new_activated_cb (GtkAction *action, GnomeDbRawForm *wid)
 {
+	/* make the first entry grab the focus */
+	if (wid->priv->iter && GDA_PARAMETER_LIST (wid->priv->iter)->parameters)
+		gnome_db_basic_form_entry_grab_focus (GNOME_DB_BASIC_FORM (wid), 
+						      (GdaParameter *)
+						      GDA_PARAMETER_LIST (wid->priv->iter)->parameters->data);
+}
+
+static void
+form_activated_cb (GnomeDbRawForm *form, gpointer data)
+{
+	if (form->priv->write_mode == GNOME_DB_DATA_WIDGET_WRITE_ON_VALUE_ACTIVATED) {
+		gint row;
+
+		row = gda_data_model_iter_get_row (form->priv->iter);
+		if (row >= 0) {
+			/* write back the current row */
+			if (gda_data_proxy_row_has_changed (form->priv->proxy, row)) {
+				GError *error = NULL;
+				if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
+					gboolean discard;
+					discard = display_error_with_keep_or_discard_choice (form, error);
+					if (discard)
+						gda_data_proxy_cancel_row_changes (form->priv->proxy, row, -1);
+					g_error_free (error);
+				}
+			}
+		}
+	}
+}
+
+static void
+form_param_changed_cb (GnomeDbRawForm *form, gpointer data)
+{
+	if (form->priv->write_mode == GNOME_DB_DATA_WIDGET_WRITE_ON_VALUE_CHANGE) {
+		gint row;
+
+		row = gda_data_model_iter_get_row (form->priv->iter);
+		if (row >= 0) {
+			/* write back the current row */
+			if (gda_data_proxy_row_has_changed (form->priv->proxy, row))
+				gda_data_proxy_apply_row_changes (form->priv->proxy, row, NULL);
+		}
+	}
+}
+
+static void
+gnome_db_raw_form_init (GnomeDbRawForm *wid)
+{
+	GtkAction *action;
 	wid->priv = g_new0 (GnomeDbRawFormPriv, 1);
 
 	wid->priv->model = NULL;
 	wid->priv->proxy = NULL;
 	wid->priv->iter = NULL;
+	wid->priv->write_mode = GNOME_DB_DATA_WIDGET_WRITE_ON_DEMAND;
+
+	g_signal_connect (G_OBJECT (wid), "activated",
+			  G_CALLBACK (form_activated_cb), NULL);
+	g_signal_connect (G_OBJECT (wid), "param_changed",
+			  G_CALLBACK (form_param_changed_cb), NULL);
 
 	/* action group */
         wid->priv->actions_group = gtk_action_group_new ("Actions");
         gtk_action_group_add_actions (wid->priv->actions_group, ui_actions, G_N_ELEMENTS (ui_actions), wid);
+	action = gtk_action_group_get_action (wid->priv->actions_group, "ActionNew");
+	g_signal_connect (G_OBJECT (action), "activate",
+			  G_CALLBACK (action_new_activated_cb), wid);
 }
 
 /**
@@ -212,6 +279,8 @@ gnome_db_raw_form_dispose (GObject *object)
 		if (form->priv->iter) {
 			g_signal_handlers_disconnect_by_func (form->priv->iter,
 							      G_CALLBACK (iter_row_changed_cb), form);
+			g_signal_handlers_disconnect_by_func (form->priv->iter,
+							      G_CALLBACK (iter_row_to_change_cb), form);
 			g_object_unref (form->priv->iter);
 			form->priv->iter = NULL;
 		}
@@ -241,9 +310,9 @@ gnome_db_raw_form_dispose (GObject *object)
 
 static void
 gnome_db_raw_form_set_property (GObject *object,
-			    guint param_id,
-			    const GValue *value,
-			    GParamSpec *pspec)
+				guint param_id,
+				const GValue *value,
+				GParamSpec *pspec)
 {
 	GnomeDbRawForm *form;
 	gpointer ptr;
@@ -252,13 +321,16 @@ gnome_db_raw_form_set_property (GObject *object,
         if (form->priv) {
                 switch (param_id) {
 		case PROP_MODEL:
-			ptr = g_value_get_pointer (value);
+			ptr = GDA_DATA_MODEL(g_value_get_object (value));
 			if (ptr)
 				g_return_if_fail (GDA_IS_DATA_MODEL (ptr));
 			if (form->priv->proxy) {
 				/* remove old data model settings */
 				g_signal_handlers_disconnect_by_func (form->priv->iter,
 								      G_CALLBACK (iter_row_changed_cb), form);
+				g_signal_handlers_disconnect_by_func (form->priv->iter,
+								      G_CALLBACK (iter_row_to_change_cb), form);
+
 				g_object_unref (G_OBJECT (form->priv->iter));
 				form->priv->iter = NULL;
 				
@@ -285,6 +357,8 @@ gnome_db_raw_form_set_property (GObject *object,
 				form->priv->iter = gda_data_model_create_iter ((GdaDataModel *) form->priv->proxy);
 				gda_data_model_iter_set_at_row (form->priv->iter, 0);
 
+				g_signal_connect (form->priv->iter, "row_to_change",
+						  G_CALLBACK (iter_row_to_change_cb), form);
 				g_signal_connect (form->priv->iter, "row_changed",
 						  G_CALLBACK (iter_row_changed_cb), form);
 
@@ -301,6 +375,7 @@ gnome_db_raw_form_set_property (GObject *object,
 				gnome_db_raw_form_initialize (form, NULL, NULL);
 			}
 
+			gnome_db_raw_form_widget_set_write_mode ((GnomeDbDataWidget *) form, form->priv->write_mode);
 			g_signal_emit_by_name (object, "proxy_changed", form->priv->proxy);
 			break;
 		default:
@@ -321,13 +396,38 @@ gnome_db_raw_form_get_property (GObject *object,
         form = GNOME_DB_RAW_FORM (object);
         if (form->priv) {
                 switch (param_id) {
+		case PROP_MODEL:
+			g_value_set_object(value, form->priv->model);					
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 			break;
-                }
+		}
         }	
 }
 
+static gboolean
+iter_row_to_change_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
+{
+	gboolean allow_change = TRUE;
+	if (row >= 0) {
+		if (form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) {
+			/* write back the current row */
+			if (gda_data_proxy_row_has_changed (form->priv->proxy, row)) {
+				GError *error = NULL;
+				if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
+					allow_change = display_error_with_keep_or_discard_choice (form, 
+												  error);
+					if (allow_change)
+						gda_data_proxy_cancel_row_changes (form->priv->proxy, row, -1);
+					g_error_free (error);
+				}
+			}
+		}
+	}
+	
+	return allow_change;
+}
+
 static void
 iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
 {
@@ -344,8 +444,10 @@ iter_row_changed_cb (GdaDataModelIter *iter, gint row, GnomeDbRawForm *form)
 			param = (GdaParameter *) params->data;
 			attributes = gda_data_proxy_get_value_attributes (form->priv->proxy, row, 
 							       gda_data_model_iter_get_column_for_param (iter, param));
-			gnome_db_basic_form_entry_set_sensitive ((GnomeDbBasicForm *) form, 
-								 param, !(attributes & GDA_VALUE_ATTR_NO_MODIF));
+			gnome_db_basic_form_entry_set_editable ((GnomeDbBasicForm *) form, 
+								param, !(attributes & GDA_VALUE_ATTR_NO_MODIF));
+			gnome_db_basic_form_entry_show ((GnomeDbBasicForm *) form, 
+							param, !(attributes & GDA_VALUE_ATTR_UNUSED));
 			params = params->next;
 		}
 	}
@@ -437,6 +539,10 @@ action_new_cb (GtkAction *action, GnomeDbRawForm *form)
 	GError *error = NULL;
 	GSList *list;
 
+	if (form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) 
+		if (! gda_data_model_iter_can_be_moved (form->priv->iter))
+			return;
+
 	/* append a row in the proxy */
 	newrow = gda_data_model_append_row (GDA_DATA_MODEL (form->priv->proxy), &error);
 	if (newrow == -1) {
@@ -452,7 +558,7 @@ action_new_cb (GtkAction *action, GnomeDbRawForm *form)
 	list = GDA_PARAMETER_LIST (form->priv->iter)->parameters;
 	while (list) {
 		GdaParameter *param;
-		const GdaValue *value;
+		const GValue *value;
 		
 		g_object_get (G_OBJECT (list->data), "full_bind", &param, NULL);
 		if (! param) {
@@ -460,12 +566,11 @@ action_new_cb (GtkAction *action, GnomeDbRawForm *form)
 									    GDA_PARAMETER (list->data));
 			gda_parameter_set_value (GDA_PARAMETER (list->data), value);
 		}
+		else
+			g_object_unref (param);
 		
 		list = g_slist_next (list);
 	}
-	
-	gnome_db_basic_form_set_entries_default (GNOME_DB_BASIC_FORM (form));
-	gnome_db_basic_form_set_entries_auto_default (GNOME_DB_BASIC_FORM (form), TRUE);
 }
 
 static void
@@ -476,6 +581,15 @@ action_delete_cb (GtkAction *action, GnomeDbRawForm *form)
 	row = gda_data_model_iter_get_row (form->priv->iter);
 	g_return_if_fail (row >= 0);
 	gda_data_proxy_delete (form->priv->proxy, row);
+
+	if (form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) {
+		/* force the proxy to apply the current row deletion */
+		gint newrow;
+
+		newrow = gda_data_model_iter_get_row (form->priv->iter);
+		if (row == newrow) /* => row has been marked as delete but nit yet really deleted */
+			gda_data_proxy_apply_row_changes (form->priv->proxy, row, NULL);
+	}
 }
 
 static void
@@ -491,7 +605,32 @@ action_undelete_cb (GtkAction *action, GnomeDbRawForm *form)
 static void
 action_commit_cb (GtkAction *action, GnomeDbRawForm *form)
 {
-	gda_data_proxy_apply_all_changes (form->priv->proxy, NULL);
+	gint row;
+
+	row = gda_data_model_iter_get_row (form->priv->iter);
+	if (form->priv->write_mode >= GNOME_DB_DATA_WIDGET_WRITE_ON_ROW_CHANGE) {
+		gint newrow;
+		GError *error = NULL;
+
+		if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
+			GtkWidget *dlg;
+
+			dlg = create_data_error_dialog (form, FALSE, error);
+			gtk_dialog_run (GTK_DIALOG (dlg));
+			gtk_widget_destroy (dlg);
+			g_error_free (error);
+		}
+		else {
+			newrow = gda_data_model_iter_get_row (form->priv->iter);
+			if (row != newrow) /* => current row has changed because the proxy had to emit a "row_removed" when
+					      actually succedded the commit 
+					      => we need to come back to that row
+					   */
+				gda_data_model_iter_set_at_row (form->priv->iter, row);
+		}
+	}
+	else
+		gda_data_proxy_apply_all_changes (form->priv->proxy, NULL);
 }
 
 static void
@@ -674,3 +813,73 @@ gnome_db_raw_form_widget_set_gda_model (GnomeDbDataWidget *iface, GdaDataModel *
 
 	g_object_set (form, "model", model, NULL);
 }
+
+static gboolean
+gnome_db_raw_form_widget_set_write_mode (GnomeDbDataWidget *iface, GnomeDbDataWidgetWriteMode mode)
+{
+	GnomeDbRawForm *form;
+	
+	g_return_val_if_fail (GNOME_DB_IS_RAW_FORM (iface), FALSE);
+	form = GNOME_DB_RAW_FORM (iface);
+	g_return_val_if_fail (form->priv, FALSE);
+
+	form->priv->write_mode = mode;
+	return TRUE;
+}
+
+static GnomeDbDataWidgetWriteMode
+gnome_db_raw_form_widget_get_write_mode (GnomeDbDataWidget *iface)
+{
+	GnomeDbRawForm *form;
+	
+	g_return_val_if_fail (GNOME_DB_IS_RAW_FORM (iface), GNOME_DB_DATA_WIDGET_WRITE_ON_DEMAND);
+	form = GNOME_DB_RAW_FORM (iface);
+	g_return_val_if_fail (form->priv, GNOME_DB_DATA_WIDGET_WRITE_ON_DEMAND);
+
+	return form->priv->write_mode;
+}
+
+/*
+ * Errors reporting
+ */
+static GtkWidget *
+create_data_error_dialog (GnomeDbRawForm *form, gboolean with_question, GError *filled_error)
+{
+	GtkWidget *dlg;
+	gchar *msg;
+
+	if (with_question)
+		msg = g_strdup_printf ("<b>%s:</b>\n\n%s",
+				       _("Current modified data is invalid"),
+				       _("You may now choose to correct it, or to discard\n"
+					 "the modifications.\n\n"
+					 "What do you wan to do?"));
+	else
+		msg = g_strdup_printf ("<b>%s:</b>\n\n%s",
+				       _("Current modified data is invalid"),
+				       _("please correct it and try again, or discard\n"
+					 "the modifications."));
+	dlg = gtk_message_dialog_new_with_markup ((GtkWindow *) gtk_widget_get_toplevel (GTK_WIDGET (form)), GTK_DIALOG_MODAL,
+						  GTK_MESSAGE_ERROR, 
+						  with_question ? GTK_BUTTONS_NONE : GTK_BUTTONS_CLOSE,
+						  msg);
+	g_free (msg);
+	return dlg;
+}
+
+/*
+ * Returns: TRUE if current data can be discarded
+ */
+static gboolean
+display_error_with_keep_or_discard_choice (GnomeDbRawForm *form, GError *filled_error)
+{
+	GtkWidget *dlg;
+	gint res;
+	dlg = create_data_error_dialog (form, TRUE, filled_error);
+	gtk_dialog_add_buttons (GTK_DIALOG (dlg),
+				_("Discard modified data"), GTK_RESPONSE_REJECT,
+				_("Correct data first"), GTK_RESPONSE_NONE, NULL);
+	res = gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_widget_destroy (dlg);
+	return (res == GTK_RESPONSE_REJECT) ? TRUE : FALSE;
+}
