@@ -47,10 +47,11 @@ static void     model_store_data (Module *module, GtkTreeIter *iter);
 enum
 {
 	C_SELECTION_CHANGED,
+	C_ROW_ACTIVATED,
 	LAST_SIGNAL
 };
 
-static gint gnome_db_selector_signals[LAST_SIGNAL] = { 0 };
+static gint gnome_db_selector_signals[LAST_SIGNAL] = { 0, 0 };
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
@@ -95,8 +96,17 @@ gnome_db_selector_class_init (GnomeDbSelectorClass * class)
 			      NULL, NULL,
 			      marshal_VOID__OBJECT, G_TYPE_NONE, 1,
 			      G_TYPE_OBJECT);
+	gnome_db_selector_signals[C_ROW_ACTIVATED] =
+		g_signal_new ("row_activated",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GnomeDbSelectorClass, row_activated),
+			      NULL, NULL,
+			      marshal_VOID__OBJECT, G_TYPE_NONE, 1,
+			      G_TYPE_OBJECT);
 
 	class->selection_changed = NULL;
+	class->row_activated = NULL;
 	object_class->dispose = gnome_db_selector_dispose;
 }
 
@@ -194,7 +204,8 @@ object_weak_notify (GnomeDbSelector *mgsel, GObject *obj)
 		/* Tell that we don't need to weak unref the Ref object */
 		mgsel->priv->ref_object = NULL;
 }
-
+static void tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, 
+					GtkTreeViewColumn *column, GnomeDbSelector *mgsel);
 static void tree_selection_changed_cb (GtkTreeSelection *select, GnomeDbSelector *mgsel);
 static void
 gnome_db_selector_dispose (GObject *object)
@@ -202,7 +213,7 @@ gnome_db_selector_dispose (GObject *object)
 	GnomeDbSelector *mgsel;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_SELECTOR (object));
+	g_return_if_fail (GNOME_DB_IS_SELECTOR (object));
 	mgsel = GNOME_DB_SELECTOR (object);
 
 	if (mgsel->priv) {
@@ -216,6 +227,8 @@ gnome_db_selector_dispose (GObject *object)
 		}
 
 		if (mgsel->priv->treeview) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (mgsel->priv->treeview),
+							      G_CALLBACK (tree_view_row_activated_cb), mgsel);
 			select = gtk_tree_view_get_selection (GTK_TREE_VIEW (mgsel->priv->treeview));
 			g_signal_handlers_disconnect_by_func (G_OBJECT (select),
 							      G_CALLBACK (tree_selection_changed_cb), mgsel);
@@ -549,6 +562,9 @@ gnome_db_selector_initialize (GnomeDbSelector *mgsel, GObject *ref_object, gbool
 		}
 
 		/* selection signal */
+		g_signal_connect (G_OBJECT (treeview), "row-activated",
+				  G_CALLBACK (tree_view_row_activated_cb), mgsel);
+
 		select = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 		gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
 		g_signal_connect (G_OBJECT (select), "changed",
@@ -600,6 +616,28 @@ tree_value_set_func (GtkTreeViewColumn *tree_column,
         g_list_free (renderers);
 }
 
+static void
+tree_view_row_activated_cb (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, GnomeDbSelector *mgsel)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	GObject *sel_obj = NULL;
+
+	model = gtk_tree_view_get_model (tree_view);
+	if (!gtk_tree_model_get_iter (model, &iter, path))
+		return;
+	
+	gtk_tree_model_get (model, &iter, OBJ_COLUMN, &sel_obj, -1);
+
+#ifdef debug_signal
+	g_print (">> 'ROW_ACTIVATED' from %s\n", __FUNCTION__);
+#endif
+	g_signal_emit (G_OBJECT (mgsel), gnome_db_selector_signals[C_ROW_ACTIVATED], 0, sel_obj);
+#ifdef debug_signal
+	g_print ("<< 'ROW_ACTIVATED' from %s\n", __FUNCTION__);
+#endif
+}
+
 static void
 tree_selection_changed_cb (GtkTreeSelection *select, GnomeDbSelector *mgsel) 
 {
@@ -621,7 +659,6 @@ tree_selection_changed_cb (GtkTreeSelection *select, GnomeDbSelector *mgsel)
 #ifdef debug_signal
 	g_print ("<< 'SELECTION_CHANGED' from %s\n", __FUNCTION__);
 #endif
-
 }
 
 
@@ -643,7 +680,7 @@ gnome_db_selector_set_mode_columns (GnomeDbSelector *mgsel, GObject *ref_object,
 {
 	gboolean keep_model, keep_columns;
 
-	g_return_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel));
+	g_return_if_fail (mgsel && GNOME_DB_IS_SELECTOR (mgsel));
 	g_return_if_fail (mgsel->priv);
 
 	keep_model = (ref_object == mgsel->priv->ref_object) && (mgsel->priv->mode == mode);
@@ -693,7 +730,7 @@ gnome_db_selector_set_selected_object (GnomeDbSelector *mgsel, GObject *selectio
         GtkTreeIter iter;
 	gpointer obj;
 	
-        g_return_val_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel), FALSE);
+        g_return_val_if_fail (mgsel && GNOME_DB_IS_SELECTOR (mgsel), FALSE);
         g_return_val_if_fail (selection && G_IS_OBJECT (selection), FALSE);
 
         model = mgsel->priv->model;
@@ -786,7 +823,7 @@ get_cut_path_depth (const gchar *path, guint depth)
 void
 gnome_db_selector_set_headers_visible (GnomeDbSelector *mgsel, gboolean visible)
 {
-	g_return_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel));
+	g_return_if_fail (mgsel && GNOME_DB_IS_SELECTOR (mgsel));
 	g_return_if_fail (mgsel->priv);
 
 	mgsel->priv->headers_visible = visible;
@@ -807,7 +844,7 @@ gnome_db_selector_set_column_label (GnomeDbSelector *mgsel, guint column, const
 {
 	GtkTreeViewColumn *tcolumn;
 
-	g_return_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel));
+	g_return_if_fail (mgsel && GNOME_DB_IS_SELECTOR (mgsel));
 	g_return_if_fail (mgsel->priv);
 
 	tcolumn = gtk_tree_view_get_column (mgsel->priv->treeview, column);
@@ -823,7 +860,7 @@ gnome_db_selector_set_column_label (GnomeDbSelector *mgsel, guint column, const
 void
 gnome_db_selector_expand_all (GnomeDbSelector *mgsel)
 {
-	g_return_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel));
+	g_return_if_fail (mgsel && GNOME_DB_IS_SELECTOR (mgsel));
 	g_return_if_fail (mgsel->priv);
 
 	if (mgsel->priv->treeview)
@@ -839,7 +876,7 @@ gnome_db_selector_expand_all (GnomeDbSelector *mgsel)
 void
 gnome_db_selector_collapse_all (GnomeDbSelector *mgsel)
 {
-	g_return_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel));
+	g_return_if_fail (mgsel && GNOME_DB_IS_SELECTOR (mgsel));
 	g_return_if_fail (mgsel->priv);
 
 	if (mgsel->priv->treeview)
@@ -864,7 +901,7 @@ gnome_db_selector_get_selected_object (GnomeDbSelector *mgsel)
 	GtkTreeSelection *select = NULL;
 	GObject *sel_obj = NULL;
 
-	g_return_val_if_fail (mgsel && IS_GNOME_DB_SELECTOR (mgsel), NULL);
+	g_return_val_if_fail (mgsel && GNOME_DB_IS_SELECTOR (mgsel), NULL);
 	g_return_val_if_fail (mgsel->priv, NULL);
 
 	if (mgsel->priv->treeview)
