@@ -1,6 +1,6 @@
 /* gnome-db-canvas-db-relations.c
  *
- * Copyright (C) 2002 - 2004 Vivien Malerba
+ * Copyright (C) 2002 - 2006 Vivien Malerba
  * Copyright (C) 2002 Fernando Martins
  *
  * This program is free software; you can redistribute it and/or
@@ -21,17 +21,12 @@
 
 #include <gtk/gtk.h>
 #include "marshal.h"
-#include <libgnomedb/gnome-db-database.h>
-#include <libgnomedb/gnome-db-table.h>
-#include <libgnomedb/gnome-db-table-field.h>
-#include <libgnomedb/gnome-db-field.h>
-#include <libgnomedb/gnome-db-constraint.h>
-#include <libgnomedb/gnome-db-ref-base.h>
+#include <libgda/libgda.h>
+#include <glib/gi18n-lib.h>
 #include "gnome-db-canvas-db-relations.h"
 #include "gnome-db-canvas-entity.h"
 #include "gnome-db-canvas-field.h"
 #include "gnome-db-canvas-fkconstraint.h"
-#include "gnome-db-graph-item.h"
 
 static void gnome_db_canvas_db_relations_class_init (GnomeDbCanvasDbRelationsClass * class);
 static void gnome_db_canvas_db_relations_init       (GnomeDbCanvasDbRelations * canvas);
@@ -40,13 +35,13 @@ static void gnome_db_canvas_db_relations_finalize   (GObject   * object);
 /* virtual functions */
 static void       create_canvas_items (GnomeDbCanvas *canvas);
 static void       clean_canvas_items  (GnomeDbCanvas *canvas);
-static void       graph_item_added    (GnomeDbCanvas *canvas, GnomeDbGraphItem *item);
-static void       graph_item_dropped  (GnomeDbCanvas *canvas, GnomeDbGraphItem *item);
+static void       graph_item_added    (GnomeDbCanvas *canvas, GdaGraphItem *item);
+static void       graph_item_dropped  (GnomeDbCanvas *canvas, GdaGraphItem *item);
 static GtkWidget *build_context_menu  (GnomeDbCanvas *canvas);
 static void       set_pixels_per_unit (GnomeDbCanvas *canvas, gdouble n);
 
-static void       db_nullified_cb     (GnomeDbDatabase *db, GnomeDbCanvas *canvas);
-static void       db_constraint_added_cb (GnomeDbDatabase *db, GnomeDbConstraint *cstr, GnomeDbCanvas *canvas);
+static void       db_destroyed_cb     (GdaDictDatabase *db, GnomeDbCanvas *canvas);
+static void       db_constraint_added_cb (GdaDictDatabase *db, GdaDictConstraint *cstr, GnomeDbCanvas *canvas);
 
 static void       drag_action_dcb     (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCanvasItem *to_item);
 
@@ -55,8 +50,8 @@ static GObjectClass *parent_class = NULL;
 
 /* HASH functions for the key of type FKTables */
 typedef struct {
-	GnomeDbTable *fk_table;
-	GnomeDbTable *ref_pk_table;
+	GdaDictTable *fk_table;
+	GdaDictTable *ref_pk_table;
 } FKTables;
 
 static guint
@@ -95,10 +90,10 @@ g_fktables_equal (gconstpointer a, gconstpointer b)
 struct _GnomeDbCanvasDbRelationsPrivate
 {
 	GSList      *ent_items;
-	GHashTable  *hash_tables; /* key = GnomeDbTable, value = GnomeDbCanvasEntity */
+	GHashTable  *hash_tables; /* key = GdaDictTable, value = GnomeDbCanvasEntity */
 	GHashTable  *hash_constraints; /* key = FKTables, value = GnomeDbCanvasFkconstrains */
 
-	GnomeDbDatabase  *db;
+	GdaDictDatabase  *db;
 };
 
 GType
@@ -158,14 +153,14 @@ gnome_db_canvas_db_relations_finalize (GObject *object)
 	GnomeDbCanvasDbRelations *canvas;
 
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS_DB_RELATIONS (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS_DB_RELATIONS (object));
 
 	canvas = GNOME_DB_CANVAS_DB_RELATIONS (object);
 
 	if (canvas->priv) {
 		if (canvas->priv->db) {
 			g_signal_handlers_disconnect_by_func (G_OBJECT (canvas->priv->db),
-							      G_CALLBACK (db_nullified_cb), canvas);
+							      G_CALLBACK (db_destroyed_cb), canvas);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (canvas->priv->db),
 							      G_CALLBACK (db_constraint_added_cb), canvas);
 		}
@@ -226,7 +221,7 @@ drag_action_question_dlg (GnomeDbCanvas *canvas, GSList *cstr_list)
 		GSList *pairs, *plist;
 		gint nb_pairs, i;
 
-		pairs = gnome_db_constraint_fkey_get_fields (GNOME_DB_CONSTRAINT (list->data));
+		pairs = gda_dict_constraint_fkey_get_fields (GDA_DICT_CONSTRAINT (list->data));
 		nb_pairs = g_slist_length (pairs);
 
 		button = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (bgroup), 
@@ -248,12 +243,12 @@ drag_action_question_dlg (GnomeDbCanvas *canvas, GSList *cstr_list)
 		plist = pairs;
 		i = 0;
 		while (plist) {
-			GnomeDbConstraintFkeyPair *pair = GNOME_DB_CONSTRAINT_FK_PAIR (pairs->data);
+			GdaDictConstraintFkeyPair *pair = GDA_DICT_CONSTRAINT_FK_PAIR (pairs->data);
 			gchar *str;
 
 			str = g_strdup_printf ("%s.%s", 
-					       gnome_db_base_get_name (GNOME_DB_BASE (gnome_db_field_get_entity (GNOME_DB_FIELD (pair->fkey)))),
-					       gnome_db_base_get_name (GNOME_DB_BASE (pair->fkey)));
+					       gda_object_get_name (GDA_OBJECT (gda_entity_field_get_entity (GDA_ENTITY_FIELD (pair->fkey)))),
+					       gda_object_get_name (GDA_OBJECT (pair->fkey)));
 			label = gtk_label_new (str);
 			g_free (str);
 			gtk_table_attach (GTK_TABLE (table), label, 0, 1, i, i+1, 0, 0, 0, 0);
@@ -264,8 +259,8 @@ drag_action_question_dlg (GnomeDbCanvas *canvas, GSList *cstr_list)
 			gtk_widget_show (label);
 			
 			str = g_strdup_printf ("%s.%s", 
-					       gnome_db_base_get_name (GNOME_DB_BASE (gnome_db_field_get_entity (GNOME_DB_FIELD (pair->ref_pkey)))),
-					       gnome_db_base_get_name (GNOME_DB_BASE (pair->ref_pkey)));
+					       gda_object_get_name (GDA_OBJECT (gda_entity_field_get_entity (GDA_ENTITY_FIELD (pair->ref_pkey)))),
+					       gda_object_get_name (GDA_OBJECT (pair->ref_pkey)));
 			label = gtk_label_new (str);
 			g_free (str);
 			gtk_table_attach (GTK_TABLE (table), label, 2, 3, i, i+1, 0, 0, 0, 0);
@@ -287,44 +282,44 @@ drag_action_question_dlg (GnomeDbCanvas *canvas, GSList *cstr_list)
 static void
 drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCanvasItem *to_item)
 {
-	GnomeDbField *fk_field = NULL, *ref_pk_field = NULL;
-	GnomeDbTable *fk_table, *ref_pk_table;
+	GdaEntityField *fk_field = NULL, *ref_pk_field = NULL;
+	GdaDictTable *fk_table, *ref_pk_table;
 	GSList *cstr_list;
 	gboolean create_constraint = FALSE;
 
-	if (IS_GNOME_DB_CANVAS_FIELD (from_item))
+	if (GNOME_DB_IS_CANVAS_FIELD (from_item))
 		fk_field = gnome_db_canvas_field_get_field (GNOME_DB_CANVAS_FIELD (from_item));
-	if (IS_GNOME_DB_CANVAS_FIELD (to_item))
+	if (GNOME_DB_IS_CANVAS_FIELD (to_item))
 		ref_pk_field = gnome_db_canvas_field_get_field (GNOME_DB_CANVAS_FIELD (to_item));
 
 	/* dismiss if we don't have two fields */
-	if (!fk_field || !IS_GNOME_DB_TABLE_FIELD (fk_field) || 
-	    !ref_pk_field || !IS_GNOME_DB_TABLE_FIELD (ref_pk_field))
+	if (!fk_field || !GDA_IS_DICT_FIELD (fk_field) || 
+	    !ref_pk_field || !GDA_IS_DICT_FIELD (ref_pk_field))
 		return;
 
-	fk_table = GNOME_DB_TABLE (gnome_db_field_get_entity (fk_field));
-	ref_pk_table = GNOME_DB_TABLE (gnome_db_field_get_entity (ref_pk_field));
+	fk_table = GDA_DICT_TABLE (gda_entity_field_get_entity (fk_field));
+	ref_pk_table = GDA_DICT_TABLE (gda_entity_field_get_entity (ref_pk_field));
 
-	cstr_list = gnome_db_database_get_tables_fk_constraints (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->db,
+	cstr_list = gda_dict_database_get_tables_fk_constraints (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->db,
 							   fk_table, ref_pk_table, FALSE);
 	if (cstr_list) {
 		/* already some existing constraints */
 		GSList *clist;
-		GnomeDbConstraint *found = NULL;
+		GdaDictConstraint *found = NULL;
 
 		/* see if a constraint already has that pair of fields */
 		clist = cstr_list;
 		while (clist && !found) {
 			GSList *pairs, *plist;
 
-			pairs = gnome_db_constraint_fkey_get_fields (GNOME_DB_CONSTRAINT (clist->data));
+			pairs = gda_dict_constraint_fkey_get_fields (GDA_DICT_CONSTRAINT (clist->data));
 			plist = pairs;
 			while (plist) {
 				if (!found) {
-					GnomeDbConstraintFkeyPair *pair = GNOME_DB_CONSTRAINT_FK_PAIR (pairs->data);
-					if ((((GnomeDbField*) pair->fkey == fk_field) && ((GnomeDbField*) pair->ref_pkey == ref_pk_field)) ||
-					    (((GnomeDbField*) pair->fkey == ref_pk_field) && ((GnomeDbField*) pair->ref_pkey == fk_field)))
-						found = GNOME_DB_CONSTRAINT (clist->data);
+					GdaDictConstraintFkeyPair *pair = GDA_DICT_CONSTRAINT_FK_PAIR (pairs->data);
+					if ((((GdaEntityField*) pair->fkey == fk_field) && ((GdaEntityField*) pair->ref_pkey == ref_pk_field)) ||
+					    (((GdaEntityField*) pair->fkey == ref_pk_field) && ((GdaEntityField*) pair->ref_pkey == fk_field)))
+						found = GDA_DICT_CONSTRAINT (clist->data);
 				}
 				g_free (plist->data);
 				plist = g_slist_next (plist);
@@ -338,7 +333,7 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
 			/* simply display a message */
 			GtkWidget *dlg;
 
-			if (gnome_db_constraint_get_table (found) == fk_table)
+			if (gda_dict_constraint_get_table (found) == fk_table)
 				dlg = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
 									  GTK_BUTTONS_CLOSE,
 									  _("<big>A constraint already exists</big>\n"
@@ -363,7 +358,7 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
 			/* keep only the constraints where the table is the FK_table */
 			list = cstr_list;
 			while (list) {
-				if (gnome_db_constraint_get_table (GNOME_DB_CONSTRAINT (list->data)) == fk_table)
+				if (gda_dict_constraint_get_table (GDA_DICT_CONSTRAINT (list->data)) == fk_table)
 					clist = g_slist_append (clist, list->data);
 				list = g_slist_next (list);
 			}
@@ -375,7 +370,7 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
 			if (response == GTK_RESPONSE_ACCEPT) {
 				GtkWidget *rb = g_object_get_data (G_OBJECT (dlg), "rb");
 				GSList *group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (rb));
-				GnomeDbConstraint *add_to = NULL;
+				GdaDictConstraint *add_to = NULL;
 
 				while (group && !add_to) {
 					if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (group->data))) {
@@ -386,15 +381,15 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
 				
 				if (add_to) {
 					GSList *nlist;
-					GnomeDbConstraintFkeyPair *pair;
+					GdaDictConstraintFkeyPair *pair;
 
-					nlist = gnome_db_constraint_fkey_get_fields (add_to);
-					pair = g_new0 (GnomeDbConstraintFkeyPair, 1);
-					pair->fkey = GNOME_DB_TABLE_FIELD (fk_field);
-					pair->ref_pkey = GNOME_DB_TABLE_FIELD (ref_pk_field);
+					nlist = gda_dict_constraint_fkey_get_fields (add_to);
+					pair = g_new0 (GdaDictConstraintFkeyPair, 1);
+					pair->fkey = GDA_DICT_FIELD (fk_field);
+					pair->ref_pkey = GDA_DICT_FIELD (ref_pk_field);
 					pair->ref_pkey_repl = NULL;
 					nlist = g_slist_append (nlist, pair);
-					gnome_db_constraint_fkey_set_fields (add_to, nlist);
+					gda_dict_constraint_fkey_set_fields (add_to, nlist);
 					
 					/* memory libreation */
 					list = nlist;
@@ -420,20 +415,20 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
 	/* actually create the constraint */
 	if (create_constraint) {
 		GSList *nlist, *list;
-		GnomeDbConstraint *cstr;
-		GnomeDbConstraintFkeyPair *pair;
+		GdaDictConstraint *cstr;
+		GdaDictConstraintFkeyPair *pair;
 
-		cstr = GNOME_DB_CONSTRAINT (gnome_db_constraint_new (fk_table, CONSTRAINT_FOREIGN_KEY));
+		cstr = GDA_DICT_CONSTRAINT (gda_dict_constraint_new (fk_table, CONSTRAINT_FOREIGN_KEY));
 		g_object_set (G_OBJECT (cstr), "user_constraint", TRUE, NULL);
 		
 		/* add FK pair */
-		pair = g_new0 (GnomeDbConstraintFkeyPair, 1);
-		pair->fkey = GNOME_DB_TABLE_FIELD (fk_field);
-		pair->ref_pkey = GNOME_DB_TABLE_FIELD (ref_pk_field);
+		pair = g_new0 (GdaDictConstraintFkeyPair, 1);
+		pair->fkey = GDA_DICT_FIELD (fk_field);
+		pair->ref_pkey = GDA_DICT_FIELD (ref_pk_field);
 		pair->ref_pkey_repl = NULL;
 
 		nlist = g_slist_append (NULL, pair);
-		gnome_db_constraint_fkey_set_fields (cstr, nlist);
+		gda_dict_constraint_fkey_set_fields (cstr, nlist);
 		
 		/* memory libreation */
 		list = nlist;
@@ -443,18 +438,18 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
 		}
 		g_slist_free (nlist);
 
-		gnome_db_database_add_constraint (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->db, cstr);
+		gda_dict_database_add_constraint (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->db, cstr);
 		g_object_unref (G_OBJECT (cstr));
 	}
 }
 
 /**
  * gnome_db_canvas_db_relations_new
- * @dict: a #GnomeDbDict object
- * @graph: a #GnomeDbGraph object, or %NULL
+ * @dict: a #GdaDict object
+ * @graph: a #GdaGraph object, or %NULL
  *
  * Creates a new canvas widget to display the relations between the database's tables.
- * The database is the one managed by the #GnomeDbDict object which @graph refers.
+ * The database is the one managed by the #GdaDict object which @graph refers.
  *
  * @graph contains all the table's graphical representations and their respective
  * locations on the canvas, or can be %NULL (in which case nothing is displayed)
@@ -462,26 +457,26 @@ drag_action_dcb (GnomeDbCanvas *canvas, GnomeDbCanvasItem *from_item, GnomeDbCan
  * Returns: a new #GnomeDbCanvasDbRelations widget
  */
 GtkWidget *
-gnome_db_canvas_db_relations_new (GnomeDbDict *dict, GnomeDbGraph *graph)
+gnome_db_canvas_db_relations_new (GdaDict *dict, GdaGraph *graph)
 {
 	GObject *obj;
-	GnomeDbDatabase *db;
+	GdaDictDatabase *db;
 	GnomeDbCanvasDbRelations *canvas;
 
-	g_return_val_if_fail (!dict || IS_GNOME_DB_DICT (dict), NULL);
+	g_return_val_if_fail (!dict || GDA_IS_DICT (dict), NULL);
 	if (graph) {
-		g_return_val_if_fail (IS_GNOME_DB_GRAPH (graph), NULL);
-		g_return_val_if_fail (ASSERT_DICT (dict) == gnome_db_base_get_dict (GNOME_DB_BASE (graph)), NULL);
+		g_return_val_if_fail (GDA_IS_GRAPH (graph), NULL);
+		g_return_val_if_fail (ASSERT_DICT (dict) == gda_object_get_dict (GDA_OBJECT (graph)), NULL);
 	}
 
         obj = g_object_new (GNOME_DB_TYPE_CANVAS_DB_RELATIONS, "aa", FALSE, NULL);
 	gnome_canvas_set_center_scroll_region (GNOME_CANVAS (obj), TRUE);
 	canvas = GNOME_DB_CANVAS_DB_RELATIONS (obj);
 
-	/* connecting to the GnomeDbDatabase */
-	db = gnome_db_dict_get_database (ASSERT_DICT (dict));
+	/* connecting to the GdaDictDatabase */
+	db = gda_dict_get_database (ASSERT_DICT (dict));
 	canvas->priv->db = db;
-	gnome_db_base_connect_nullify (db, G_CALLBACK (db_nullified_cb), obj);
+	gda_object_connect_destroy (db, G_CALLBACK (db_destroyed_cb), obj);
 	g_signal_connect (G_OBJECT (db), "constraint_added",
 			  G_CALLBACK (db_constraint_added_cb), obj);
 
@@ -493,12 +488,12 @@ gnome_db_canvas_db_relations_new (GnomeDbDict *dict, GnomeDbGraph *graph)
 
 
 static void
-db_nullified_cb (GnomeDbDatabase *db, GnomeDbCanvas *canvas)
+db_destroyed_cb (GdaDictDatabase *db, GnomeDbCanvas *canvas)
 {
 	/* disconnecting signals */
 	GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->db = NULL;
 	g_signal_handlers_disconnect_by_func (G_OBJECT (db),
-					      G_CALLBACK (db_nullified_cb), canvas);
+					      G_CALLBACK (db_destroyed_cb), canvas);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (db),
 					      G_CALLBACK (db_constraint_added_cb), canvas);
 
@@ -508,20 +503,20 @@ db_nullified_cb (GnomeDbDatabase *db, GnomeDbCanvas *canvas)
 
 static void canvas_fkconstraint_destroy_cb (GnomeDbCanvasItem *canvas_item, GnomeDbCanvasDbRelations *canvas);
 static void
-db_constraint_added_cb (GnomeDbDatabase *db, GnomeDbConstraint *cstr, GnomeDbCanvas *canvas)
+db_constraint_added_cb (GdaDictDatabase *db, GdaDictConstraint *cstr, GnomeDbCanvas *canvas)
 {
 	FKTables key;
 	GnomeDbCanvasItem *canvas_fk;
 	GnomeCanvasItem *root, *canvas_item;
 
 	/* ignore non FK constraints */
-	if (gnome_db_constraint_get_constraint_type (cstr) != CONSTRAINT_FOREIGN_KEY)
+	if (gda_dict_constraint_get_constraint_type (cstr) != CONSTRAINT_FOREIGN_KEY)
 		return;
 
 	root = GNOME_CANVAS_ITEM (gnome_canvas_root (GNOME_CANVAS (canvas)));
 
-	key.fk_table = gnome_db_constraint_get_table (cstr);
-	key.ref_pk_table = gnome_db_constraint_fkey_get_ref_table (cstr);
+	key.fk_table = gda_dict_constraint_get_table (cstr);
+	key.ref_pk_table = gda_dict_constraint_fkey_get_ref_table (cstr);
 	canvas_fk = g_hash_table_lookup (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->hash_constraints, &key);
 	
 	if (canvas_fk) 
@@ -557,18 +552,18 @@ canvas_fkconstraint_destroy_cb (GnomeDbCanvasItem *canvas_item, GnomeDbCanvasDbR
 }
 
 /*
- * Add all the required GnomeCanvasItem objects for the associated #GnomeDbGraph object 
+ * Add all the required GnomeCanvasItem objects for the associated #GdaGraph object 
  */
 static void
 create_canvas_items (GnomeDbCanvas *canvas)
 {
 	GSList *list, *graph_items;
-	GnomeDbGraph *graph = gnome_db_canvas_get_graph (canvas);
+	GdaGraph *graph = gnome_db_canvas_get_graph (canvas);
 
-	graph_items = gnome_db_graph_get_items (graph);
+	graph_items = gda_graph_get_items (graph);
 	list = graph_items;
 	while (list) {
-		graph_item_added (canvas, GNOME_DB_GRAPH_ITEM (list->data));
+		graph_item_added (canvas, GDA_GRAPH_ITEM (list->data));
 		list = g_slist_next (list);
 	}
 	g_slist_free (graph_items);
@@ -595,13 +590,13 @@ static GtkWidget *canvas_entity_popup_func (GnomeDbCanvasEntity *ce);
  * Add the GnomeDbCanvasEntity corresponding to the graph item
  */
 static void
-graph_item_added (GnomeDbCanvas *canvas, GnomeDbGraphItem *item)
+graph_item_added (GnomeDbCanvas *canvas, GdaGraphItem *item)
 {
 	GnomeCanvasItem *canvas_item, *root;
-	GnomeDbBase *ref_obj = gnome_db_graph_item_get_ref_object (item);
+	GdaObject *ref_obj = gda_graph_item_get_ref_object (item);
 
 	root = GNOME_CANVAS_ITEM (gnome_canvas_root (GNOME_CANVAS (canvas)));
-	if (IS_GNOME_DB_TABLE (ref_obj)) {
+	if (GDA_IS_DICT_TABLE (ref_obj)) {
 		GSList *fklist, *list;
 
 		/* GnomeDbCanvasEntity for the table */
@@ -618,16 +613,16 @@ graph_item_added (GnomeDbCanvas *canvas, GnomeDbGraphItem *item)
 		gnome_canvas_update_now (GNOME_CANVAS (canvas)); /* forced here because of a GnomeCanvas bug */
 
 		/* GnomeDbCanvasFkconstraint object(s) for the FK constraints using that table */
-		fklist = gnome_db_database_get_tables_fk_constraints (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->db,
-								GNOME_DB_TABLE (ref_obj), NULL, FALSE);
+		fklist = gda_dict_database_get_tables_fk_constraints (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->db,
+								GDA_DICT_TABLE (ref_obj), NULL, FALSE);
 		list = fklist;
 		while (list) {
 			FKTables key;
-			GnomeDbConstraint *fkcons = GNOME_DB_CONSTRAINT (list->data);
+			GdaDictConstraint *fkcons = GDA_DICT_CONSTRAINT (list->data);
 			GnomeDbCanvasItem *canvas_fk;
 
-			key.fk_table = gnome_db_constraint_get_table (fkcons);
-			key.ref_pk_table = gnome_db_constraint_fkey_get_ref_table (fkcons);
+			key.fk_table = gda_dict_constraint_get_table (fkcons);
+			key.ref_pk_table = gda_dict_constraint_fkey_get_ref_table (fkcons);
 			canvas_fk = g_hash_table_lookup (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->hash_constraints, &key);
 			
 			if (canvas_fk)
@@ -676,10 +671,10 @@ canvas_entity_popup_func (GnomeDbCanvasEntity *ce)
 static void
 popup_func_delete_cb (GtkMenuItem *mitem, GnomeDbCanvasEntity *ce)
 {
-	GnomeDbGraphItem *gitem;
+	GdaGraphItem *gitem;
 
 	gitem = gnome_db_canvas_item_get_graph_item (GNOME_DB_CANVAS_ITEM (ce));
-	gnome_db_base_nullify (GNOME_DB_BASE (gitem));
+	gda_object_destroy (GDA_OBJECT (gitem));
 }
 
 
@@ -687,11 +682,11 @@ popup_func_delete_cb (GtkMenuItem *mitem, GnomeDbCanvasEntity *ce)
  * Remove the GnomeDbCanvasEntity corresponding to the graph item
  */
 static void
-graph_item_dropped (GnomeDbCanvas *canvas, GnomeDbGraphItem *item)
+graph_item_dropped (GnomeDbCanvas *canvas, GdaGraphItem *item)
 {
-	GnomeDbBase *ref_obj = gnome_db_graph_item_get_ref_object (item);
+	GdaObject *ref_obj = gda_graph_item_get_ref_object (item);
 
-	if (IS_GNOME_DB_TABLE (ref_obj)) {
+	if (GDA_IS_DICT_TABLE (ref_obj)) {
 		GtkObject *canvas_item = g_hash_table_lookup (GNOME_DB_CANVAS_DB_RELATIONS (canvas)->priv->hash_tables,
 							      ref_obj);
 		if (canvas_item) {
@@ -708,11 +703,11 @@ build_context_menu (GnomeDbCanvas *canvas)
 	GtkWidget *menu, *mitem, *submenu, *submitem;
 	GSList *tables, *list;
 	GnomeDbCanvasDbRelations *dbrels = GNOME_DB_CANVAS_DB_RELATIONS (canvas);
-	GnomeDbRefBase *refbase;
-	GnomeDbDict *dict = gnome_db_base_get_dict (GNOME_DB_BASE (dbrels->priv->db));
+	GdaObjectRef *refbase;
+	GdaDict *dict = gda_object_get_dict (GDA_OBJECT (dbrels->priv->db));
 	gboolean other_tables = FALSE;
 
-	tables = gnome_db_database_get_tables (dbrels->priv->db);
+	tables = gda_dict_database_get_tables (dbrels->priv->db);
 
 	menu = gtk_menu_new ();
 	submitem = gtk_menu_item_new_with_label (_("Add table"));
@@ -725,18 +720,18 @@ build_context_menu (GnomeDbCanvas *canvas)
 		gtk_widget_show (submenu);
 		
 		/* add a menu item for each table not currently shown displayed.
-		 * WARNING: if a GnomeDbTable is detroyed while that menu is displayed, it can still be selected
+		 * WARNING: if a GdaDictTable is detroyed while that menu is displayed, it can still be selected
 		 * from within the menu even though it will not do anything.
 		 */
 		list = tables;
 		while (list) {
 			if (! g_hash_table_lookup (dbrels->priv->hash_tables, list->data)) {
-				mitem = gtk_menu_item_new_with_label (gnome_db_base_get_name (GNOME_DB_BASE (list->data)));
+				mitem = gtk_menu_item_new_with_label (gda_object_get_name (GDA_OBJECT (list->data)));
 				gtk_widget_show (mitem);
 				gtk_menu_append (submenu, mitem);
 				
-				refbase = GNOME_DB_REF_BASE (gnome_db_ref_base_new (dict));
-				gnome_db_ref_base_set_ref_object (refbase, GNOME_DB_BASE (list->data));
+				refbase = GDA_OBJECT_REF (gda_object_ref_new (dict));
+				gda_object_ref_set_ref_object (refbase, GDA_OBJECT (list->data));
 				g_object_set_data_full (G_OBJECT (mitem), "table", refbase, g_object_unref);
 				
 				g_signal_connect (G_OBJECT (mitem), "activate", G_CALLBACK (popup_add_table_cb), canvas);
@@ -756,18 +751,18 @@ build_context_menu (GnomeDbCanvas *canvas)
 static void
 popup_add_table_cb (GtkMenuItem *mitem, GnomeDbCanvasDbRelations *canvas)
 {
-	GnomeDbRefBase *refbase;
-	GnomeDbTable *table;
+	GdaObjectRef *refbase;
+	GdaDictTable *table;
 
 	refbase = g_object_get_data (G_OBJECT (mitem), "table");
-	table = (GnomeDbTable *) gnome_db_ref_base_get_ref_object (refbase);
+	table = (GdaDictTable *) gda_object_ref_get_ref_object (refbase);
 
 	if (table) {
-		GnomeDbGraphItem *gitem;
+		GdaGraphItem *gitem;
 
-		gitem = GNOME_DB_GRAPH_ITEM (gnome_db_graph_item_new (gnome_db_base_get_dict (GNOME_DB_BASE (canvas->priv->db)), GNOME_DB_BASE (table)));
-		gnome_db_graph_item_set_position (gitem, GNOME_DB_CANVAS (canvas)->xmouse, GNOME_DB_CANVAS (canvas)->ymouse);
-		gnome_db_graph_add_item (gnome_db_canvas_get_graph (GNOME_DB_CANVAS (canvas)), gitem);
+		gitem = GDA_GRAPH_ITEM (gda_graph_item_new (gda_object_get_dict (GDA_OBJECT (canvas->priv->db)), GDA_OBJECT (table)));
+		gda_graph_item_set_position (gitem, GNOME_DB_CANVAS (canvas)->xmouse, GNOME_DB_CANVAS (canvas)->ymouse);
+		gda_graph_add_item (gnome_db_canvas_get_graph (GNOME_DB_CANVAS (canvas)), gitem);
 		g_object_unref (G_OBJECT (gitem));
 	}
 }
@@ -779,7 +774,7 @@ set_pixels_per_unit (GnomeDbCanvas *canvas, gdouble n)
 
 	list = GNOME_CANVAS_GROUP (GNOME_CANVAS (canvas)->root)->item_list;
 	while (list) {
-		if (IS_GNOME_DB_CANVAS_ENTITY (list->data))
+		if (GNOME_DB_IS_CANVAS_ENTITY (list->data))
 			g_object_set (G_OBJECT (list->data), "scale", n, NULL);
 		list = g_list_next (list);
 	}
