@@ -32,10 +32,10 @@ G_BEGIN_DECLS
  *
  */
 
-#define GNOME_DB_CANVAS_FKCONSTRAINT_TYPE          (gnome_db_canvas_fkconstraint_get_type())
+#define GNOME_DB_TYPE_CANVAS_FKCONSTRAINT          (gnome_db_canvas_fkconstraint_get_type())
 #define GNOME_DB_CANVAS_FKCONSTRAINT(obj)          G_TYPE_CHECK_INSTANCE_CAST (obj, gnome_db_canvas_fkconstraint_get_type(), GnomeDbCanvasFkconstraint)
 #define GNOME_DB_CANVAS_FKCONSTRAINT_CLASS(klass)  G_TYPE_CHECK_CLASS_CAST (klass, gnome_db_canvas_fkconstraint_get_type (), GnomeDbCanvasFkconstraintClass)
-#define IS_GNOME_DB_CANVAS_FKCONSTRAINT(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gnome_db_canvas_fkconstraint_get_type ())
+#define GNOME_DB_IS_CANVAS_FKCONSTRAINT(obj)       G_TYPE_CHECK_INSTANCE_TYPE (obj, gnome_db_canvas_fkconstraint_get_type ())
 
 
 typedef struct _GnomeDbCanvasFkconstraint        GnomeDbCanvasFkconstraint;
@@ -60,7 +60,7 @@ struct _GnomeDbCanvasFkconstraintClass
 /* generic widget's functions */
 GType      gnome_db_canvas_fkconstraint_get_type       (void);
 
-void       gnome_db_canvas_fkconstraint_add_constraint (GnomeDbCanvasFkconstraint *cc, GnomeDbConstraint *fkcons);
+void       gnome_db_canvas_fkconstraint_add_constraint (GnomeDbCanvasFkconstraint *cc, GdaDictConstraint *fkcons);
 
 G_END_DECLS
 
