@@ -1,6 +1,6 @@
 /* gnome-db-canvas-item.c
  *
- * Copyright (C) 2002 - 2005 Vivien Malerba
+ * Copyright (C) 2002 - 2006 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,25 +19,26 @@
  */
 
 #include <gtk/gtk.h>
+#include <libgda/libgda.h>
 #include "marshal.h"
 #include "gnome-db-decl.h"
 #include "gnome-db-canvas-item.h"
 #include "gnome-db-canvas-tip.h"
 #include "gnome-db-canvas-cursor.h"
-#include "gnome-db-graph-item.h"
+
 
 static void gnome_db_canvas_item_class_init (GnomeDbCanvasItemClass * class);
 static void gnome_db_canvas_item_init       (GnomeDbCanvasItem * item);
 static void gnome_db_canvas_item_finalize    (GObject   * object);
 
-static void gnome_db_canvas_item_set_property    (GObject              *object,
-					    guint                 param_id,
-					    const GValue         *value,
-					    GParamSpec           *pspec);
-static void gnome_db_canvas_item_get_property    (GObject              *object,
-					    guint                 param_id,
-					    GValue               *value,
-					    GParamSpec           *pspec);
+static void gnome_db_canvas_item_set_property    (GObject *object,
+					    guint param_id,
+					    const GValue *value,
+					    GParamSpec *pspec);
+static void gnome_db_canvas_item_get_property    (GObject *object,
+					    guint param_id,
+					    GValue *value,
+					    GParamSpec *pspec);
 static void m_drag_action  (GnomeDbCanvasItem *citem, GnomeDbCanvasItem *dragged_from, GnomeDbCanvasItem * dragged_to);
 
 struct _GnomeDbCanvasItemPrivate
@@ -48,7 +49,7 @@ struct _GnomeDbCanvasItemPrivate
 	gboolean            allow_move;
 	gboolean            allow_drag;
 	gchar              *tooltip_text;
-	GnomeDbGraphItem        *graph_item;
+	GdaGraphItem        *graph_item;
 };
 
 enum
@@ -172,7 +173,7 @@ m_drag_action  (GnomeDbCanvasItem *citem, GnomeDbCanvasItem *dragged_from, Gnome
 
 	/* make sure the parent GnomeDbCanvasItem forwards the DND info... */
 	g_object_get (G_OBJECT (citem), "parent", &parent, NULL);
-	if (IS_GNOME_DB_CANVAS_ITEM (parent)) {
+	if (GNOME_DB_IS_CANVAS_ITEM (parent)) {
 #ifdef debug_signal
 		g_print (">> 'DRAG_ACTION' from %s::%s()\n", __FILE__, __FUNCTION__);
 #endif
@@ -202,20 +203,20 @@ gnome_db_canvas_item_init (GnomeDbCanvasItem * item)
 			  G_CALLBACK (item_event), NULL);
 }
 
-static void graph_item_nullified_cb (GnomeDbGraphItem *item, GnomeDbCanvasItem *citem);
-static void graph_item_moved_cb (GnomeDbGraphItem *item, GnomeDbCanvasItem *citem);
+static void graph_item_destroyed_cb (GdaGraphItem *item, GnomeDbCanvasItem *citem);
+static void graph_item_moved_cb (GdaGraphItem *item, GnomeDbCanvasItem *citem);
 
 static void
 gnome_db_canvas_item_finalize (GObject   * object)
 {
 	GnomeDbCanvasItem *citem;
 	g_return_if_fail (object != NULL);
-	g_return_if_fail (IS_GNOME_DB_CANVAS_ITEM (object));
+	g_return_if_fail (GNOME_DB_IS_CANVAS_ITEM (object));
 	
 	citem = GNOME_DB_CANVAS_ITEM (object);
 	if (citem->priv) {
 		if (citem->priv->graph_item) 
-			graph_item_nullified_cb (citem->priv->graph_item, citem);
+			graph_item_destroyed_cb (citem->priv->graph_item, citem);
 
 		if (citem->priv->tooltip_text) 
 			g_free (citem->priv->tooltip_text);
@@ -229,10 +230,10 @@ gnome_db_canvas_item_finalize (GObject   * object)
 }
 
 static void 
-gnome_db_canvas_item_set_property    (GObject              *object,
-				guint                 param_id,
-				const GValue         *value,
-				GParamSpec           *pspec)
+gnome_db_canvas_item_set_property    (GObject *object,
+				guint param_id,
+				const GValue *value,
+				GParamSpec *pspec)
 {
 	GnomeDbCanvasItem *citem;
 	const gchar *str;
@@ -266,15 +267,15 @@ gnome_db_canvas_item_set_property    (GObject              *object,
 			return;
 
 		if (citem->priv->graph_item) 
-			graph_item_nullified_cb (citem->priv->graph_item, citem);
+			graph_item_destroyed_cb (citem->priv->graph_item, citem);
 
 		if (ptr) {
-			g_return_if_fail (IS_GNOME_DB_GRAPH_ITEM (ptr));
-			gnome_db_base_connect_nullify (ptr, G_CALLBACK (graph_item_nullified_cb), citem);
+			g_return_if_fail (GDA_IS_GRAPH_ITEM (ptr));
+			gda_object_connect_destroy (ptr, G_CALLBACK (graph_item_destroyed_cb), citem);
 			g_signal_connect (G_OBJECT (ptr), "moved",
 					  G_CALLBACK (graph_item_moved_cb), citem);
 
-			/* set the position according to the GnomeDbGraphItem */
+			/* set the position according to the GdaGraphItem */
 			citem->priv->graph_item = ptr;
 			graph_item_moved_cb (citem->priv->graph_item, citem);
 		}
@@ -282,10 +283,10 @@ gnome_db_canvas_item_set_property    (GObject              *object,
 }
 
 static void
-gnome_db_canvas_item_get_property    (GObject              *object,
-				guint                 param_id,
-				GValue               *value,
-				GParamSpec           *pspec)
+gnome_db_canvas_item_get_property    (GObject *object,
+				guint param_id,
+				GValue *value,
+				GParamSpec *pspec)
 {
 	TO_IMPLEMENT;
 }
@@ -301,7 +302,7 @@ gnome_db_canvas_item_get_property    (GObject              *object,
 GnomeDbCanvas *
 gnome_db_canvas_item_get_canvas (GnomeDbCanvasItem *item)
 {
-	g_return_val_if_fail (item && IS_GNOME_DB_CANVAS_ITEM (item), NULL);
+	g_return_val_if_fail (item && GNOME_DB_IS_CANVAS_ITEM (item), NULL);
 	return (GnomeDbCanvas *) GNOME_CANVAS_ITEM (item)->canvas;
 }
 
@@ -309,38 +310,38 @@ gnome_db_canvas_item_get_canvas (GnomeDbCanvasItem *item)
  * gnome_db_canvas_item_get_graph_item
  * @item: a #GnomeDbCanvasItem object
  *
- * Get the associated #GnomeDbGraphItem to @item.
+ * Get the associated #GdaGraphItem to @item.
  *
- * Returns: the #GnomeDbGraphItem, or %NULL
+ * Returns: the #GdaGraphItem, or %NULL
  */
-GnomeDbGraphItem *
+GdaGraphItem *
 gnome_db_canvas_item_get_graph_item (GnomeDbCanvasItem *item)
 {
-	g_return_val_if_fail (item && IS_GNOME_DB_CANVAS_ITEM (item), NULL);
+	g_return_val_if_fail (item && GNOME_DB_IS_CANVAS_ITEM (item), NULL);
 	g_return_val_if_fail (item->priv, NULL);
 
 	return item->priv->graph_item;
 }
 
 static void
-graph_item_nullified_cb (GnomeDbGraphItem *item, GnomeDbCanvasItem *citem)
+graph_item_destroyed_cb (GdaGraphItem *item, GnomeDbCanvasItem *citem)
 {
 	g_assert (citem->priv->graph_item == item);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
-					      G_CALLBACK (graph_item_nullified_cb), citem);
+					      G_CALLBACK (graph_item_destroyed_cb), citem);
 	g_signal_handlers_disconnect_by_func (G_OBJECT (item),
 					      G_CALLBACK (graph_item_moved_cb), citem);
 	citem->priv->graph_item = NULL;
 }
 
 static void
-graph_item_moved_cb (GnomeDbGraphItem *item, GnomeDbCanvasItem *citem)
+graph_item_moved_cb (GdaGraphItem *item, GnomeDbCanvasItem *citem)
 {
 	gdouble x, y;
 	gdouble xold, yold;
 
 	g_assert (citem->priv->graph_item == item);
-	gnome_db_graph_item_get_position (item, &x, &y);
+	gda_graph_item_get_position (item, &x, &y);
 	/*g_object_set (G_OBJECT (citem), "x", x, "y", y, NULL); => DOES NOT WORK */
 	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (citem), &xold, &yold, NULL, NULL);
 	gnome_canvas_item_move (GNOME_CANVAS_ITEM (citem), x-xold, y-yold);
@@ -372,7 +373,7 @@ item_event (GnomeDbCanvasItem *citem, GdkEvent *event, gpointer data)
 		/* Drag management */
 		ci = g_object_get_data (G_OBJECT (gnome_canvas_root (GNOME_CANVAS_ITEM (citem)->canvas)),
 					  "dragged_from");
-		if (ci && IS_GNOME_DB_CANVAS_ITEM (ci)) {
+		if (ci && GNOME_DB_IS_CANVAS_ITEM (ci)) {
 #ifdef debug_signal
 			g_print (">> 'DRAG_ACTION' from %s::%s()\n", __FILE__, __FUNCTION__);
 #endif
@@ -469,7 +470,7 @@ item_event (GnomeDbCanvasItem *citem, GdkEvent *event, gpointer data)
 			if (citem->priv->graph_item) {
 				gdouble x, y;
 				gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (citem), &x , &y, NULL, NULL);
-				gnome_db_graph_item_set_position (citem->priv->graph_item, x, y);
+				gda_graph_item_set_position (citem->priv->graph_item, x, y);
 			}
 #ifdef debug_signal
 			g_print (">> 'MOVED' from %s::item_event()\n", __FILE__);
@@ -484,7 +485,7 @@ item_event (GnomeDbCanvasItem *citem, GdkEvent *event, gpointer data)
 		if (citem->priv->moving) {
 			GdkEventMotion *evm = (GdkEventMotion *) (event);
 
-			g_assert (IS_GNOME_DB_CANVAS_ITEM (citem));
+			g_assert (GNOME_DB_IS_CANVAS_ITEM (citem));
 			gnome_canvas_item_move (GNOME_CANVAS_ITEM (citem), 
 						(double) evm->x - citem->priv->xstart, 
 						(double) evm->y - citem->priv->ystart);
